/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aIO.h                                                     */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Definition of a platform-independent I/O layer.    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef _aIO_H_
#define _aIO_H_

#include "aErr.h"
#include "aDefs.h"
#include "aOSDefs.h"
#include "aIOOSExport.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * opaque reference to the I/O library
 */

typedef aLIBREF aIOLib;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * I/O library manipulation routines
 */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

aIO_EXPORT aLIBRETURN aIO_GetLibRef(aIOLib* pIORef, 
				    aErr* pErr);

aIO_EXPORT aLIBRETURN aIO_ReleaseLibRef(aIOLib ioRef, 
					aErr* pErr);

aIO_EXPORT aLIBRETURN aIO_GetVersion(aIOLib ioRef, 
			             unsigned long* pVersion,
				     aErr* pErr)
	aIO_TRAP(aIOGetVersionTrap);


#ifdef __cplusplus 
}
#endif /* __cplusplus */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * opaque reference to a file
 */

typedef void* aFileRef;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Maximum number of filename bytes.
 */

#define aFILE_NAMEMAXCHARS	31

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Serial Input buffer size.
 */

#define aSERIAL_INBUFSIZE     1024

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Maximum number of TCP/IP connections on a server socket
 */

#define aTCP_MAXCONNECT         10


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * File I/O modes.
 */

typedef enum aFileMode {
  aFileModeReadOnly,
  aFileModeWriteOnly
} aFileMode;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * File storage areas.
 */

typedef enum aFileArea {
  aFileAreaUser,
  aFileAreaSystem,
  aFileAreaObject,
  aFileAreaBinary,
  aFileAreaTest,
  aFileAreaInclude,
  aFileAreaSource
} aFileArea;

#define txtFileAreaUser		"aUser"
#define txtFileAreaSystem	"aSystem"
#define txtFileAreaObject	"aObject"
#define txtFileAreaBinary	"aBinary"
#define txtFileAreaTest		"aTest"
#define txtFileAreaInclude	"aInclude"
#define txtFileAreaSource	"aSource"

#define nMAXFILEAREACHARS	8

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * File Routine definitions
 */

#ifdef __cplusplus
extern "C" {
#endif

aIO_EXPORT aLIBRETURN aFile_Open(aIOLib ioRef,
				 const char* pFilename,
				 const aFileMode eMode,
				 const aFileArea eArea,
				 aFileRef* pFileRef,
				 aErr* pErr)
		aIO_TRAP(aFileOpenTrap);

aIO_EXPORT aLIBRETURN aFile_Close(aIOLib ioRef,
				  aFileRef fileRef,
				  aErr* pErr)
		aIO_TRAP(aFileCloseTrap);

aIO_EXPORT aLIBRETURN aFile_Read(aIOLib ioRef,
				 aFileRef fileRef,
				 char* pBuffer,
				 const unsigned long nLength,
				 unsigned long* pActuallyRead,
				 aErr* pErr)
		aIO_TRAP(aFileReadTrap);

aIO_EXPORT aLIBRETURN aFile_Write(aIOLib ioRef,
				  aFileRef fileRef,
				  const char* pBuffer,
				  const unsigned long nLength,
  				  unsigned long* pActuallyWritten,
  				  aErr* pErr)
  		aIO_TRAP(aFileWriteTrap);

aIO_EXPORT aLIBRETURN aFile_Seek(aIOLib ioRef,
				 aFileRef fileRef,
				 const long nOffset,
				 aBool bFromStart,
				 aErr* pErr)
		aIO_TRAP(aFileSeekTrap);

aIO_EXPORT aLIBRETURN aFile_GetSize(aIOLib ioRef,
				    aFileRef fileRef,
				    unsigned long* pSize,
				    aErr* pErr)
		aIO_TRAP(aFileGetSizeTrap);

aIO_EXPORT aLIBRETURN aFile_Delete(aIOLib ioRef,
				   const char *pFilename,
				   const aFileArea eArea,
				   aErr* pErr) 
		aIO_TRAP(aFileDeleteTrap);
#ifdef __cplusplus 
}
#endif


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Low level operation proc defs for I/O streams.  These need to 
 * be implimented for new stream types.
 */

typedef aErr (*aStreamGetProc)(char* pData,
			       void* ref);

typedef aErr (*aStreamPutProc)(char* pData,
			       void* ref);

typedef aErr (*aStreamDeleteProc)(void* ref);

typedef void* aStreamRef;

#define aStreamLibRef(aStreamRef) (*(aIOLib*)aStreamRef)


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Stream operations.
 */

#ifdef __cplusplus
extern "C" {
#endif

aIO_EXPORT aLIBRETURN aStream_Create(aIOLib ioRef, 
				     aStreamGetProc getProc,
				     aStreamPutProc putProc,
				     aStreamDeleteProc deleteProc,
				     const void* procRef,
				     aStreamRef* pStreamRef,
				     aErr* pErr) 
		aIO_TRAP(aStreamCreateTrap);

aIO_EXPORT aLIBRETURN aStream_CreateFileInput(aIOLib ioRef, 
					      const char *pFilename,
					      const aFileArea eArea,
					      aStreamRef* pStreamRef,
					      aErr* pErr) 
		aIO_TRAP(aStreamCreateFileInputTrap);

aIO_EXPORT aLIBRETURN aStream_CreateFileOutput(aIOLib ioRef, 
					       const char *pFilename,
				 	       const aFileArea eArea,
					       aStreamRef* pStreamRef,
					       aErr* pErr)
		aIO_TRAP(aStreamCreateFileOutputTrap);

aIO_EXPORT aLIBRETURN aStream_CreateSerial(aIOLib ioRef, 
					   const char *pPortName, 
					   const unsigned int nBaudRate, 
					   aStreamRef* pStreamRef, 
					   aErr* pErr)
		aIO_TRAP(aStreamCreateSerialTrap);

aIO_EXPORT aLIBRETURN aStream_CreateSocket(aIOLib ioRef, 
					   const unsigned long address, 
					   const unsigned short port, 
					   const aBool bServer,
					   aStreamRef* pStreamRef, 
					   aErr* pErr)
		aIO_TRAP(aStreamCreateSocketTrap);

aIO_EXPORT aLIBRETURN aStream_Read(aIOLib ioRef, 
				   aStreamRef streamRef,
				   char* pBuffer,
    				   const unsigned long nLength,
    				   aErr* pErr)
		aIO_TRAP(aStreamReadTrap);

aIO_EXPORT aLIBRETURN aStream_Write(aIOLib ioRef, 
				    aStreamRef streamRef,
				    const char* pBuffer,
    				    const unsigned long nLength,
    				    aErr* pErr)
		aIO_TRAP(aStreamWriteTrap);

aIO_EXPORT aLIBRETURN aStream_ReadLine(aIOLib ioRef, 
				       aStreamRef streamRef,
				       char* pBuffer,
    				       const unsigned long nMaxLength,
    				       aErr* pErr)
		aIO_TRAP(aStreamReadLineTrap);

aIO_EXPORT aLIBRETURN aStream_WriteLine(aIOLib ioRef, 
					aStreamRef streamRef,
				        const char* pBuffer,
				        aErr* pErr)
		aIO_TRAP(aStreamWriteLineTrap);

aIO_EXPORT aLIBRETURN aStream_Destroy(aIOLib ioRef, 
				     aStreamRef streamRef,
				     aErr* pErr)
		aIO_TRAP(aStreamDestroyTrap);

#ifdef __cplusplus 
}
#endif


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Memory Pool allocator routines
 */

#ifdef __cplusplus
extern "C" {
#endif

typedef void* aMemPoolRef;

aIO_EXPORT aLIBRETURN aMemPool_Create(aIOLib ioRef,
				      aMemSize objectSize,
		     		      aMemSize blockSize,
		     		      aMemPoolRef* pPoolRef,
		     		      aErr* pErr)
		aIO_TRAP(aMemPoolCreateTrap);

aIO_EXPORT aLIBRETURN aMemPool_Alloc(aIOLib ioRef,
				     aMemPoolRef poolRef,
		    		     void** ppObj,
		     		     aErr* pErr)
		aIO_TRAP(aMemPoolAllocTrap);

aIO_EXPORT aLIBRETURN aMemPool_Free(aIOLib ioRef,
				    aMemPoolRef poolRef,
		   		    void* pObj,
		     		    aErr* pErr)
		aIO_TRAP(aMemPoolFreeTrap);

aIO_EXPORT aLIBRETURN aMemPool_Destroy(aIOLib ioRef,
				       aMemPoolRef poolRef,
		     		       aErr* pErr)
		aIO_TRAP(aMemPoolDestroyTrap);

#ifdef __cplusplus 
}
#endif

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Symbol table routines
 */

#ifdef __cplusplus
extern "C" {
#endif

typedef void* aSymbolTableRef;

#define aMAXIDENTIFIERLEN		32

typedef aErr (*symDataDeleteProc)(void* pData, void* ref);

aIO_EXPORT aLIBRETURN aSymbolTable_Create(aIOLib ioRef,
					  aSymbolTableRef* pSymbolTableRef,
				          aErr* pErr)
		aIO_TRAP(aSymbolTableCreateTrap);

aIO_EXPORT aLIBRETURN aSymbolTable_Insert(aIOLib ioRef,
					  aSymbolTableRef symbolTableRef,
			 		  const char* identifier,
			 		  void* pData,
  			 		  symDataDeleteProc deleteProc,
  			 		  void* deleteRef,
				          aErr* pErr)
		aIO_TRAP(aSymbolTableInsertTrap);

aIO_EXPORT aLIBRETURN aSymbolTable_Find(aIOLib ioRef,
					aSymbolTableRef symbolTableRef,
		       			const char* identifier,
		       			void** ppFoundData,
				        aErr* pErr)
		aIO_TRAP(aSymbolTableFindTrap);

aIO_EXPORT aLIBRETURN aSymbolTable_Destroy(aIOLib ioRef,
					   aSymbolTableRef symbolTableRef,
				     	   aErr* pErr)
		aIO_TRAP(aSymbolTableDestroyTrap);

#ifdef __cplusplus 
}
#endif


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * token types
 */

typedef unsigned char tkType;

#define tkInt		(tkType)0
#define tkIdentifier	(tkType)1
#define tkSpecial	(tkType)2
#define tkString	(tkType)3
#define tkPreProc	(tkType)4
#define tkNewLine	(tkType)5

#define tkErrUntermCmnt		1
#define tkErrIncludeNotFnd	2
#define tkErrCompile		3
#define tkErrDuplicateDefine	4
#define tkErrBadArgumentList	5

typedef struct aToken {
  tkType		eType;
  union {
    int			integer;
    char		identifier[aMAXIDENTIFIERLEN];
    char		string[aMAXIDENTIFIERLEN];
    char		special;
  } v;
} aToken;

typedef struct aTokenInfo {
  unsigned int		nLine;
  unsigned int		nColumn;
  char*			pSourceName;
  char*			pSourceLine;
} aTokenInfo;

typedef unsigned int tkError;

typedef aErr (*aTokenErrProc)(tkError error,
			      unsigned int nLine,
			      unsigned int nColumn,
			      unsigned int nData,
			      char* data[],
			      void* errProcRef);

#ifdef __cplusplus
extern "C" {
#endif

typedef void* aTokenizerRef;

aIO_EXPORT aLIBRETURN aToken_GetInfo(aIOLib ioRef,
				     aToken* pToken,
				     aTokenInfo* pTokenInfo,
				     aErr* pErr)
		aIO_TRAP(aTokenGetInfoTrap);

aIO_EXPORT aLIBRETURN aTokenizer_Create(aIOLib ioRef,
					aStreamRef tokenStream,
			       		const char* streamName,
		       	       		aFileArea eIncludeArea,
		       	       		aTokenErrProc errProc,
		       	       		void* errProcRef,
					aTokenizerRef* pTokenizerRef,
				        aErr* pErr)
		aIO_TRAP(aTokenizerCreateTrap);

aIO_EXPORT aLIBRETURN aTokenizer_Next(aIOLib ioRef,
				      aTokenizerRef tokenizerRef,
				      aToken** ppToken,
				      aErr* pErr)
		aIO_TRAP(aTokenizerNextTrap);

aIO_EXPORT aLIBRETURN aTokenizer_PushBack(aIOLib ioRef,
				          aTokenizerRef tokenizerRef,
				          aToken* pToken,
				          aErr* pErr)
		aIO_TRAP(aTokenizerPushBackTrap);

aIO_EXPORT aLIBRETURN aTokenizer_Dispose(aIOLib ioRef,
				         aTokenizerRef tokenizerRef,
				         aToken* pToken,
				         aErr* pErr)
		aIO_TRAP(aTokenizerDisposeTrap);

aIO_EXPORT aLIBRETURN aTokenizer_Destroy(aIOLib ioRef,
					 aTokenizerRef tokenizerRef,
				     	 aErr* pErr)
		aIO_TRAP(aTokenizerDestroyTrap);

#ifdef __cplusplus 
}
#endif


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * setting file routines
 */

#ifdef __cplusplus
extern "C" {
#endif

typedef void* aSettingFileRef;

aIO_EXPORT aLIBRETURN aSettingFile_Create(aIOLib ioRef,
					  const unsigned int nMaxSettingLen,
			 		  const char* pFileName,
			 		  aSettingFileRef* pSettingFileRef,
			  		  aErr* pErr)
		aIO_TRAP(aSettingFileCreateTrap);

aIO_EXPORT aLIBRETURN aSettingFile_GetInt(aIOLib ioRef,
					  aSettingFileRef settingFileRef,
			 		  const char* key, 
			 		  int* pInt,
			 		  const int nDefault,
			  		  aErr* pErr)
		aIO_TRAP(aSettingFileGetIntTrap);
			 
aIO_EXPORT aLIBRETURN aSettingFile_GetULong(aIOLib ioRef,
					    aSettingFileRef settingFileRef,
			   		    const char* key, 
			   		    unsigned long* pULong,
			   		    const unsigned long nDefault,
			  		    aErr* pErr)
		aIO_TRAP(aSettingFileGetULongTrap);
		   
aIO_EXPORT aLIBRETURN aSettingFile_GetFloat(aIOLib ioRef,
					    aSettingFileRef settingFileRef,
			   		    const char* key, 
			   		    float* pFloat,
			   		    const float fDefault,
			  		    aErr* pErr)
		aIO_TRAP(aSettingFileGetFloatTrap);

aIO_EXPORT aLIBRETURN aSettingFile_GetString(aIOLib ioRef,
					     aSettingFileRef settingFileRef,
			    		     const char* key, 
			    		     char** ppString,
			    		     const char* pDefault,
			  		     aErr* pErr)
		aIO_TRAP(aSettingFileGetStringTrap);

aIO_EXPORT aLIBRETURN aSettingFile_GetInetAddr(aIOLib ioRef,
					       aSettingFileRef settingFileRef,
			    		       const char* key, 
			    		       unsigned long* pInetAddr,
			    		       const unsigned long pDefault,
			  		       aErr* pErr)
		aIO_TRAP(aSettingFileGetInetAddrTrap);

aIO_EXPORT aLIBRETURN aSettingFile_Destroy(aIOLib ioRef,
					   aSettingFileRef settingFileRef,
			  		   aErr* pErr)
		aIO_TRAP(aSettingFileDestroyTrap);

#ifdef __cplusplus 
}
#endif


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Common OS-independant I/O functions
 */

#ifdef __cplusplus
extern "C" {
#endif

aIO_EXPORT aLIBRETURN aIO_GetMSTicks(aIOLib ioRef,
				     unsigned long* pNTicks,
				     aErr* pErr)
		aIO_TRAP(aIOGetMSTicksTrap);

aIO_EXPORT aLIBRETURN aIO_MSSleep(aIOLib ioRef,
				  const unsigned long msTime,
				  aErr* pErr)
		aIO_TRAP(aIOMSSleepTrap);

#ifdef __cplusplus 
}
#endif


#endif /* _aIO_H_ */
