/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aFileTests.c                                              */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Testing module for the aFile shared library.       */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "aIO.h"
#include "aIOTests.h"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * local prototypes
 */

static void sTest_Print(const char* text,
			aStreamRef out);

static aBool sTest_aFile_Open(aIOLib ioLibRef, aStreamRef out);
static aBool sTest_aFile_Close(aIOLib ioLibRef, aStreamRef out);
static aBool sTest_aFile_Read(aIOLib ioLibRef, aStreamRef out);
static aBool sTest_aFile_Write(aIOLib ioLibRef, aStreamRef out);
static aBool sTest_aFile_Seek(aIOLib ioLibRef, aStreamRef out);
static aBool sTest_aFile_GetSize(aIOLib ioLibRef, aStreamRef out);
static aBool sTest_aFile_Delete(aIOLib ioLibRef, aStreamRef out);
static aBool sTest_aFile_Use(aIOLib ioLibRef, aStreamRef out);

static aBool sTest_aIO_GetMSTicks(aIOLib ioLibRef, aStreamRef out);
static aBool sTest_aIO_MSSleep(aIOLib ioLibRef, aStreamRef out);

static aBool sTest_aTokenizer_Create(aIOLib ioLibRef, aStreamRef out);

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_Write
 */

void sTest_Print(const char* text,
		 aStreamRef out)
{
  aStream_Write(aStreamLibRef(out), out, text, 
  		aStringLen(text), NULL);

} /* sTest_Print */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aFile_Open
 */

aBool sTest_aFile_Open(aIOLib ioLibRef, 
		       aStreamRef out)
{
  aErr err;
  char msg[100];
  aFileRef fileRef;
  char filename[aFILE_NAMEMAXCHARS+2];
  char testwritefilename[aFILE_NAMEMAXCHARS+2];
  aFileRef testwritefileRef;
  int i;
  unsigned long uSize;

  aStream_WriteLine(ioLibRef, out, "  aFile_Open", NULL);

#ifndef aPALM
  aStringCopy(msg, "   NULL Lib Ref (read)..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(NULL, "test", aFileModeReadOnly, aFileAreaUser,
  	     &fileRef, &err);
  if (err != aErrParam)
    return aTrue;
  if (aFile_Open(NULL, "test", aFileModeReadOnly, aFileAreaUser,
  		 &fileRef, NULL) == 0)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aStringCopy(msg, "   NULL Lib Ref (write)..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(NULL, "test", aFileModeWriteOnly, aFileAreaUser,
  	     &fileRef, &err);
  if (err != aErrParam)
    return aTrue;
  if (aFile_Open(NULL, "test", aFileModeWriteOnly, aFileAreaUser,
  		 &fileRef, NULL) == 0)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */

  /* try opening a file with a NULL file ref both with and 
   * without an error pointer in read mode
   */
  aStringCopy(msg, "   NULL File Ref (read)");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Open(ioLibRef, "test", aFileModeReadOnly, 
  		  aFileAreaUser, NULL, &err))
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, "test", aFileModeReadOnly, 
 		 aFileAreaUser, NULL, NULL) == 0)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try opening a file with a NULL file ref both with and 
   * without an error pointer in write mode
   */
  aStringCopy(msg, "   NULL File Ref (write)");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Open(ioLibRef, "test", aFileModeWriteOnly, 
  		  aFileAreaUser, NULL, &err))
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, "test", aFileModeWriteOnly, 
  		 aFileAreaUser, NULL, NULL) == 0)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try opening a file with a NULL filename in read mode
   */
  aStringCopy(msg, "   NULL File Name (read)");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Open(ioLibRef, NULL, aFileModeReadOnly, 
  		  aFileAreaUser, &fileRef, &err))
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, "test", aFileModeWriteOnly, 
  		 aFileAreaUser, NULL, NULL) == 0)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aStringCopy(msg, "   NULL File Name (write)");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Open(ioLibRef, NULL, aFileModeWriteOnly, 
  		  aFileAreaUser, &fileRef, &err))
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, "test", aFileModeWriteOnly, 
  		 aFileAreaUser, NULL, NULL) == 0)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try opening a file that doesn't exist in read mode
   */
  aStringCopy(msg, "   Non-existent file");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Open(ioLibRef, "non-existent", aFileModeReadOnly, 
  		  aFileAreaUser, &fileRef, &err))
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (err != aErrNotFound)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, "non_existent", aFileModeReadOnly, 
  		 aFileAreaUser, &fileRef, NULL) == 0)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try opening a file with a name that is too long in read mode
   */
  for (i = 0; i < aFILE_NAMEMAXCHARS + 1; i++)
    filename[i] = 'a';
  filename[i] = '\0';
  aStringCopy(msg, "   Filename too long (read)");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Open(ioLibRef, filename, aFileModeReadOnly, 
  		  aFileAreaUser, &fileRef, &err))
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (err != aErrFileNameLength)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, filename, aFileModeReadOnly, 
  		 aFileAreaUser, &fileRef, NULL) == 0)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try opening a file with a name that is too long in write mode
   */
  aStringCopy(msg, "   Filename too long (write)");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Open(ioLibRef, filename, aFileModeWriteOnly, 
  		  aFileAreaUser, &fileRef, &err))
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (err != aErrFileNameLength)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, filename, aFileModeWriteOnly, 
  		 aFileAreaUser, &fileRef, NULL) == 0)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try opening an actual file in write mode with an error pointer
   */
  aStringCopy(testwritefilename, "testwritefile");

  aStringCopy(msg, "   Open \"testwritefile\" writing");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, testwritefilename, aFileModeWriteOnly, 
  		 aFileAreaUser, &testwritefileRef, &err))
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(msg, ", closing");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Close(ioLibRef, testwritefileRef, NULL))
    return aTrue;
  aStringCopy(msg, ", deleting...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Delete(ioLibRef, testwritefilename, aFileAreaUser, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  /* open a file, write to it, close and reopen for writing, close
   * and check size, should be zero length.
   */
  aStringCopy(msg, "   Reopen for writing zero length");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, testwritefilename, aFileModeWriteOnly, 
  		 aFileAreaUser, &testwritefileRef, &err))
    return aTrue;
  aStringCopy(msg, ", writing");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Write(ioLibRef, testwritefileRef, "test", 5, NULL, NULL))
    return aTrue;
  aStringCopy(msg, ", closing");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Close(ioLibRef, testwritefileRef, NULL))
    return aTrue;
  aStringCopy(msg, ", reopen");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, testwritefilename, aFileModeWriteOnly, 
  		 aFileAreaUser, &testwritefileRef, &err))
    return aTrue;
  aStringCopy(msg, ", sizing...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_GetSize(ioLibRef, testwritefileRef, &uSize, NULL))
    return aTrue;
  if (uSize != 0)
    return aTrue;
  aStringCopy(msg, ", closing");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Close(ioLibRef, testwritefileRef, NULL))
    return aTrue;
  aStringCopy(msg, ", deleting...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Delete(ioLibRef, testwritefilename, aFileAreaUser, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  return aFalse;

} /* sTest_aFile_Open */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aFile_Close
 */

aBool sTest_aFile_Close(aIOLib ioLibRef, 
		        aStreamRef out)
{
  aErr err;
  char msg[100];
  aFileRef fileRef;
  aFileRef badFileRef;
  char testclosefilename[aFILE_NAMEMAXCHARS+2];  /* UNL I added this to help w/ consistency */

  aStream_WriteLine(ioLibRef, out, "  aFile_Close", NULL);

  aStringCopy(testclosefilename, "test_fc");  

  badFileRef=(void*)42; /* junk pointer */
  
  /* try closing a file with a NULL lib ref
   */

#ifndef aPALM
  aStringCopy(msg, "   NULL Lib Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, testclosefilename , aFileModeWriteOnly, 
  		 aFileAreaUser, &fileRef, &err))
    return aTrue;  
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Close(NULL, fileRef, &err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Close(NULL,fileRef,NULL)) /* should fail */
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err); /* good close */
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */

  /* try closing a file with a NULL file ref
   */

  aStringCopy(msg, "   NULL File Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);


  aFile_Open(ioLibRef, testclosefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Close(ioLibRef, NULL, &err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Close(ioLibRef, NULL, NULL)) /* should fail */
    return aTrue;
  aFile_Close(ioLibRef, fileRef, &err); /* good close */
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try closing a file with a BAD file ref
   * THIS CAUSED A BIG FAT CRASH
   */
/*
  aStringCopy(msg, "   BAD File Ref...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, "test_fc", aFileModeWriteOnly, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aFile_Close(ioLibRef,badFileRef,&err);
  if (err != aErrParam)
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err);
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
*/

  /* try closing a file multiple times
   * (DEBUG was catching a extra de-allocation)
   */
  aStringCopy(msg, "   Multiple Close");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testclosefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aFile_Close(ioLibRef, fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(msg, ", error on 2nd");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Close(ioLibRef, fileRef, &err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Close(ioLibRef, fileRef, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try opening and closing existing file
   */
  aStringCopy(msg, "   Existing Close...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testclosefilename, aFileModeReadOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aFile_Close(ioLibRef, fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* get rid of our working file 
   */
  aFile_Delete(ioLibRef, testclosefilename, aFileAreaUser, &err);
  if (err != aErrNone)
    return aTrue;

  return aFalse;

} /* sTest_aFile_Close */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aFile_Read
 */

aBool sTest_aFile_Read(aIOLib ioLibRef, 
		       aStreamRef out)
{
  aErr err;
  char msg[100];
  char buff[100];
  char guff[100];
  aFileRef fileRef;
  unsigned long uwCt;
  int nCmpRes;
  char testreadfilename[aFILE_NAMEMAXCHARS+2];  /* UNL I added this to help w/ consistency */

  aStringCopy(testreadfilename,"test_rd");  

  aStream_WriteLine(ioLibRef, out, "  aFile_Read", NULL);

  /* create a generic test file */
  /* bad param tests come first so write-only won't matter */
  
  aFile_Open(ioLibRef, testreadfilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(buff, "0123456789\n");
  aFile_Write(ioLibRef, fileRef, buff, aStringLen(buff), &uwCt, &err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != aStringLen(buff))
    return aTrue;

  /* try reading with NULL lib ref
   */

#ifndef aPALM  
  aStringCopy(msg, "   NULL Lib Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Read(NULL,fileRef,guff,10,&uwCt,&err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Read(NULL,fileRef,guff,10,&uwCt,NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */


  /* try reading with NULL file ref
   */

  aStringCopy(msg, "   NULL File Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Read(ioLibRef, NULL, guff, 10, &uwCt, &err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Read(ioLibRef, NULL, guff, 10, &uwCt, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  /* try reading with NULL read buffer
   */

  aStringCopy(msg, "   NULL Read Buffer");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Read(ioLibRef, fileRef, NULL, 10, &uwCt, &err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Read(ioLibRef, fileRef, NULL, 10, &uwCt, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  /* try reading from write only file
   */

  aStringCopy(msg, "   Read From Write Only");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Read(ioLibRef, fileRef, guff, 10, &uwCt, &err);
  if (err != aErrMode)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Read(ioLibRef, fileRef, guff, 10, &uwCt, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  /* close working file */
  /* and re-open file as read only */

  aFile_Close(ioLibRef, fileRef, &err);
  if (err != aErrNone)
    return aTrue;

  aFile_Open(ioLibRef, testreadfilename, aFileModeReadOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;


  /* try reading properly
   */

  aStringCopy(msg, "   Read From Read Only");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(guff, "aaaaaaaaaa");
  aFile_Read(ioLibRef, fileRef, guff, 10, &uwCt, &err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != 10)
    return aTrue;
  nCmpRes = aStringCompare(guff, "0123456789");
  if (nCmpRes) /* 0 means equal */
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Seek(ioLibRef, fileRef, 0, aTrue, &err);
  if (err != aErrNone)
    return aTrue;
  if (aFile_Read(ioLibRef, fileRef, guff, 10, &uwCt, NULL)) /* should succeed */
    return aTrue;
  if (uwCt != 10)
    return aTrue;
  nCmpRes = aStringCompare(guff, "0123456789");
  if (nCmpRes) /* 0 means equal */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aFile_Close(ioLibRef, fileRef, &err);   /* close working file */
  if (err != aErrNone)
    return aTrue;

  /* try reading from closed file
   */

  aStringCopy(msg, "   Read From Closed File");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(guff, "aaaaaaaaaa");
  aFile_Read(ioLibRef, fileRef, guff, 10, &uwCt, &err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Read(ioLibRef, fileRef, guff, 10, &uwCt, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* erase working file */
  aFile_Delete(ioLibRef, testreadfilename, aFileAreaUser, &err);
  if (err != aErrNone)
    return aTrue;

  return aFalse;
  
} /* sTest_aFile_Read */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aFile_Write
 */

aBool sTest_aFile_Write(aIOLib ioLibRef, 
		        aStreamRef out)
{

  aErr err;
  char msg[100];
  char buff[100];
  aFileRef fileRef;
  unsigned long uwCt;

  char testwritefilename[aFILE_NAMEMAXCHARS+2];  /* UNL I added this to help w/ consistency */

  aStringCopy(testwritefilename,"test_wr");  
  
  aStream_WriteLine(ioLibRef, out, "  aFile_Write", NULL);

  /* try writing with NULL lib ref
   */

#ifndef aPALM
  aStringCopy(msg, "   NULL Lib Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testwritefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(buff, "0123456789\n");
  aFile_Write(NULL,fileRef,buff,aStringLen(buff),&uwCt,&err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Write(NULL,fileRef,buff,aStringLen(buff),&uwCt,NULL)) /* should fail */
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err);   /* close working file */
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */

  /* try writing with NULL file ref
   */

  aStringCopy(msg, "   NULL File Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testwritefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(buff, "0123456789\n");
  aFile_Write(ioLibRef,NULL,buff,aStringLen(buff),&uwCt,&err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Write(ioLibRef,NULL,buff,aStringLen(buff),&uwCt,NULL)) /* should fail */
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err);   /* close working file */
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try writing with NULL buffer pointer
   */

  aStringCopy(msg, "   NULL Buffer Ptr");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testwritefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Write(ioLibRef,fileRef,NULL,0,&uwCt,&err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Write(ioLibRef,fileRef,NULL,0,&uwCt,NULL)) /* should fail */
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err);   /* close working file */
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try writing to a write-only file
   */

  aStringCopy(msg, "   Write to Write Only");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testwritefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(buff, "0123456789\n");
  aFile_Write(ioLibRef,fileRef,buff,aStringLen(buff),&uwCt,&err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != aStringLen(buff))
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Write(ioLibRef,fileRef,buff,aStringLen(buff),&uwCt,NULL)) /* should succeed */
    return aTrue;
  if (uwCt != aStringLen(buff))
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err);   /* close working file */
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try writing with an empty buffer
   */

  aStringCopy(msg, "   Write Empty Buffer");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testwritefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(buff, ""); /* empty */
  aFile_Write(ioLibRef,fileRef,buff,aStringLen(buff),&uwCt,&err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != aStringLen(buff))
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Write(ioLibRef,fileRef,buff,aStringLen(buff),&uwCt,NULL)) /* should succeed */
    return aTrue;
  if (uwCt != aStringLen(buff))
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err);   /* close working file */
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try writing with occupied buffer but 0 char
   */

  aStringCopy(msg, "   Write Zero Bytes");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testwritefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(buff, "0123456789\n");
  aFile_Write(ioLibRef,fileRef,buff,0,&uwCt,&err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != 0)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Write(ioLibRef,fileRef,buff,0,&uwCt,NULL)) /* should succeed */
    return aTrue;
  if (uwCt != 0)
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err);   /* close working file */
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  /* try writing to closed file
   */

  aStringCopy(msg, "   Write to Closed File");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(buff, "0123456789\n");
  aFile_Write(ioLibRef,fileRef,buff,0,&uwCt,&err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Write(ioLibRef,fileRef,buff,0,&uwCt,NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  
  /* try writing to a read-only file
   */

  aStringCopy(msg, "   Write to Read Only");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testwritefilename, aFileModeReadOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(buff, "0123456789\n");
  aFile_Write(ioLibRef,fileRef,buff,aStringLen(buff),&uwCt,&err);
  if (err != aErrMode)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Write(ioLibRef,fileRef,buff,aStringLen(buff),&uwCt,NULL)) /* should fail */
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err);   /* close working file */
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* NO EASY WAY TO TEST FOR aErrWrite */

  /* erase working file */
  aFile_Delete(ioLibRef, testwritefilename, aFileAreaUser, &err);
  if (err != aErrNone)
    return aTrue;
  
  return aFalse;
  
} /* sTest_aFile_Write */






/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aFile_Seek
 */

aBool sTest_aFile_Seek(aIOLib ioLibRef, 
		       aStreamRef out)
{

  aErr err;
  char msg[100];
  char buff[100];
  char guff[100];
  aFileRef fileRef;
  unsigned long uwCt;
  char testseekfilename[aFILE_NAMEMAXCHARS+2];  /* UNL I added this to help w/ consistency */

  aStringCopy(testseekfilename, "test_sk");  
  
  aStream_WriteLine(ioLibRef, out, "  aFile_Seek", NULL);

  /* create a generic test file */
  /* bad param tests come first so write-only won't matter */
  
  aFile_Open(ioLibRef, testseekfilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(buff, "0123456789\n");
  aFile_Write(ioLibRef, fileRef, buff, aStringLen(buff), &uwCt, &err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != aStringLen(buff))
    return aTrue;

  /* try seeking with NULL lib ref
   */

#ifndef aPALM
  aStringCopy(msg, "   NULL Lib Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Seek(NULL,fileRef,4,aTrue,&err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Seek(NULL, fileRef, 1, aFalse, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */

  /* try seeking with NULL file ref */

  aStringCopy(msg, "   NULL File Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Seek(ioLibRef,NULL,4,aTrue,&err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Seek(ioLibRef, NULL, 1, aFalse, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  /* close working file (11 bytes long) */
  /* and re-open file as read only */
  
  aFile_Close(ioLibRef,fileRef,&err);
  if (err != aErrNone)
    return aTrue;
  aFile_Open(ioLibRef, testseekfilename, aFileModeReadOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;

  /* try seeking normally from beginning and from current pos
   */
  aStringCopy(msg, "   Seek Normally F&B");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(guff, "aaaaaaaaaa");
  /* move to byte 4 */
  aFile_Seek(ioLibRef, fileRef, 4, aTrue, &err);
  if (err != aErrNone)
    return aTrue;
  /* advances 1 byte */
  aFile_Read(ioLibRef, fileRef, guff, 1, &uwCt, &err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != 1)
    return aTrue;
  if (guff[0] != '4')
    return aTrue;
  /* advances 1 more byte */
  aFile_Seek(ioLibRef, fileRef, 1, aFalse, &err);
  if (err != aErrNone)
    return aTrue;
  aFile_Read(ioLibRef, fileRef, guff, 1, &uwCt, &err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != 1)
    return aTrue;
  if (guff[0] != '6')
    return aTrue;
  /* back up two bytes */
  aFile_Seek(ioLibRef, fileRef, -2, aFalse, &err);
  if (err != aErrNone)
    return aTrue;
  aFile_Read(ioLibRef, fileRef, guff, 1, &uwCt, &err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != 1)
    return aTrue;
  if (guff[0] != '5')
    return aTrue;

  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(guff,"aaaaaaaaaa");
  if (aFile_Seek(ioLibRef, fileRef, 4, aTrue, &err)) /* should succeed */
    return aTrue;
  aFile_Read(ioLibRef, fileRef, guff, 1, &uwCt, &err); /* advances 1 byte */
  if (err != aErrNone)
    return aTrue;
  if (uwCt != 1)
    return aTrue;
  if (guff[0] != '4')
    return aTrue;
  if (aFile_Seek(ioLibRef, fileRef, 1, aFalse, &err)) /* should succeed */
    return aTrue; /* seek advances 1 more byte */
  aFile_Read(ioLibRef, fileRef, guff, 1, &uwCt, &err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != 1)
    return aTrue;
  if (guff[0] != '6')
    return aTrue;
  if (aFile_Seek(ioLibRef, fileRef, -2, aFalse, &err)) /* should succeed */
    return aTrue; /* seek backs up two bytes */
  aFile_Read(ioLibRef, fileRef, guff, 1, &uwCt, &err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != 1)
    return aTrue;
  if (guff[0] != '5')
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  /* now try seeking forward past EOF
   */

  aStringCopy(msg, "   Seek Forward Past EOF");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);

  aFile_Seek(ioLibRef, fileRef, 5, aTrue, &err); /* start at offset 5 */
  if (err != aErrNone)
    return aTrue;
  aStringCopy(guff,"aaaaaaaaaa");

  aFile_Seek(ioLibRef, fileRef, 20, aTrue, &err);
  if (err != aErrEOF)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);

  if (!aFile_Seek(ioLibRef, fileRef, 20, aTrue, NULL)) /* should fail */
    return aTrue;
  aFile_Read(ioLibRef, fileRef, guff, 1, &uwCt, &err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != 1)
    return aTrue;
  if (guff[0] != '5') /* must end at offset 5 */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  /* now try seeking backward past EOF
   */

  aStringCopy(msg, "   Seek Backward Past EOF");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(guff,"aaaaaaaaaa");
  aFile_Seek(ioLibRef, fileRef, 5, aTrue, &err); /* start at offset 5 */
  if (err != aErrNone)
    return aTrue;
  aFile_Seek(ioLibRef, fileRef, -20, aTrue, &err);
  if (err != aErrEOF)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Seek(ioLibRef, fileRef, -20, aTrue, NULL)) /* should fail */
    return aTrue;
  aFile_Read(ioLibRef, fileRef, guff, 1, &uwCt, &err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != 1)
    return aTrue;
  if (guff[0] != '5') /* must end at offset 5 */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  aFile_Close(ioLibRef, fileRef, &err);   /* close working file */
  if (err != aErrNone)
    return aTrue;


  /* try seeking in a closed file
   */

  aStringCopy(msg, "   Seek in Closed File");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(guff,"aaaaaaaaaa");
  aFile_Seek(ioLibRef, fileRef, 5, aTrue, &err); /* start at offset 5 */
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Seek(ioLibRef, fileRef, 5, aTrue, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* erase working file */
  aFile_Delete(ioLibRef, testseekfilename, aFileAreaUser, &err);
  if (err != aErrNone)
    return aTrue;

  return aFalse;

} /* sTest_aFile_Seek */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aFile_Seek
 */

aBool sTest_aFile_GetSize(aIOLib ioLibRef, 
		          aStreamRef out)
{

  aErr err;
  char msg[100];
  char buff[100];
  aFileRef fileRef;
  unsigned long uwCt;
  unsigned long uSize;
  char testgetsizefilename[aFILE_NAMEMAXCHARS+2];  /* UNL I added this to help w/ consistency */

  aStringCopy(testgetsizefilename,"test_gs.txt");  
 
  aStream_WriteLine(ioLibRef, out, "  aFile_GetSize", NULL);

  /* create a generic test file */
  /* bad param tests come first so write-only won't matter */
  /* (don't use slash-n so file size is consistent across platforms) */
  
  aFile_Open(ioLibRef, testgetsizefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aStringCopy(buff, "0123456789");
  aFile_Write(ioLibRef,fileRef,buff,aStringLen(buff),&uwCt,&err);
  if (err != aErrNone)
    return aTrue;
  if (uwCt != aStringLen(buff))
    return aTrue;

  /* get size with NULL lib ref
   */

#ifndef aPALM
  aStringCopy(msg, "   NULL Lib Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_GetSize(NULL,fileRef,&uSize,&err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_GetSize(NULL,fileRef,&uSize,NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */

  /* get size with NULL file ref
   */

  aStringCopy(msg, "   NULL File Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_GetSize(ioLibRef,NULL,&uSize,&err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_GetSize(ioLibRef, NULL, &uSize, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* get size for write-only file
   */

  aStringCopy(msg, "   Size of Write Only File");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  uSize=0;
  aFile_GetSize(ioLibRef,fileRef,&uSize,&err);
  if (err != aErrNone)
    return aTrue;
  if (uSize != 10)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  uSize=0;
  if (aFile_GetSize(ioLibRef,fileRef,&uSize,NULL)) /* should succeed */
    return aTrue;
  if (uSize != 10)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  /* close working file */
  /* and re-open file as read only */
  
  aFile_Close(ioLibRef,fileRef,&err);
  if (err != aErrNone)
    return aTrue;
  aFile_Open(ioLibRef, testgetsizefilename, aFileModeReadOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;

  /* get size for read-only file
   */

  aStringCopy(msg, "   Size of Read Only File");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  uSize=0;
  aFile_GetSize(ioLibRef,fileRef,&uSize,&err);
  if (err != aErrNone)
    return aTrue;
  if (uSize != 10)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  uSize=0;
  if (aFile_GetSize(ioLibRef,fileRef,&uSize,NULL)) /* should succeed */
    return aTrue;
  if (uSize != 10)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aFile_Close(ioLibRef,fileRef,&err);
  if (err != aErrNone)
    return aTrue;

  /* close working file */    

  /* get size of closed file
   */

  aStringCopy(msg, "   Size of Closed File");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  uSize=0;
  aFile_GetSize(ioLibRef,fileRef,&uSize,&err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  uSize=0;
  if (!aFile_GetSize(ioLibRef,fileRef,&uSize,NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* destroy working file */
  aFile_Delete(ioLibRef, testgetsizefilename, aFileAreaUser, &err);
  if (err != aErrNone)
    return aTrue;

  return aFalse;

} /* sTest_aFile_GetSize */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aFile_Delete
 */

aBool sTest_aFile_Delete(aIOLib ioLibRef, 
		         aStreamRef out)
{

  aErr err;
  char msg[100];
  aFileRef fileRef;
  char testdeletefilename[aFILE_NAMEMAXCHARS+2];  /* UNL I added this to help w/ consistency */

  aStringCopy(testdeletefilename,"test_del");  
 
  aStream_WriteLine(ioLibRef, out, "  aFile_Delete", NULL);

  /* try deleting with NULL lib ref */

#ifndef aPALM
  aStringCopy(msg, "   NULL Lib Ref");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef,testdeletefilename,aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  aFile_Close(ioLibRef,fileRef,&err);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Delete(NULL, testdeletefilename, aFileAreaUser, &err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Delete(NULL, testdeletefilename, aFileAreaUser, NULL)) /* should fail */
    return aTrue;
  aFile_Delete(ioLibRef, testdeletefilename, aFileAreaUser, &err);
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */

  /* try deleting with NULL file name */

  aStringCopy(msg, "   NULL File Name");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Delete(ioLibRef, NULL, aFileAreaUser, &err);
  if (err != aErrParam)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Delete(ioLibRef, NULL, aFileAreaUser, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try deleting a non-existent file */

  aStringCopy(msg, "   Delete Non-Existent File");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Delete(ioLibRef, "non_existent_file", aFileAreaUser, &err);
  if (err != aErrNotFound)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Delete(ioLibRef, "non_existent_file", aFileAreaUser, NULL)) /* should fail */
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* try to delete open file */

#if !defined(aPALM)
  aStringCopy(msg, "   Delete Open File");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aStringCopy(msg, ", error val");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testdeletefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aFile_Delete(ioLibRef, testdeletefilename, aFileAreaUser, &err);
  if (err != aErrIO)
    return aTrue;
  aStringCopy(msg, ", null error...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aFile_Delete(ioLibRef, "test_del", aFileAreaUser, NULL)) /* should fail */
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err);
  if (err != aErrNone)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */

  /* try to open deleted file (verify that it is deleted) */

  aStringCopy(msg, "   Open Deleted File...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aFile_Open(ioLibRef, testdeletefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;
  aFile_Close(ioLibRef,fileRef,&err);
  if (err != aErrNone)
    return aTrue;
  aFile_Delete(ioLibRef, testdeletefilename, aFileAreaUser, &err);
  if (err != aErrNone)
    return aTrue;
  aFile_Open(ioLibRef, testdeletefilename, aFileModeReadOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNotFound)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  return aFalse;
    
} /* sTest_aFile_Delete */




/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aFile_Use
 */

aBool sTest_aFile_Use(aIOLib ioLibRef, 
		      aStreamRef out)
{

  aErr err;
  char msg[100];
  aFileRef fileRef;
  unsigned char data[6];
  unsigned long bytes;
  char testusefilename[aFILE_NAMEMAXCHARS+2];

  aStringCopy(testusefilename,"test_use");  
 
  aStream_WriteLine(ioLibRef, out, "  aFile Usage", NULL);

  /* try opening writing and closing some known bytes */

  aStringCopy(msg, "   Usage, open, ");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);

  aFile_Open(ioLibRef, testusefilename, aFileModeWriteOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;

  aStringCopy(msg, "write, ");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);

  data[0] = 43;
  data[1] = 32;
  data[2] = 45;
  data[3] = 78;
  data[4] = 43;
  data[5] = 56;
  
  aFile_Write(ioLibRef, fileRef, (char*)data, 6, &bytes, &err);
  if (err != aErrNone)
    return aTrue;
  if (bytes != 6)
    return aTrue;

  aStringCopy(msg, "close...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);

  aFile_Close(ioLibRef, fileRef, &err);
  if (err != aErrNone)
    return aTrue;

  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* now re-open, check size, read bytes, and close */
  fileRef = NULL;
  aStringCopy(msg, "   Usage, open, ");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);

  aFile_Open(ioLibRef, testusefilename, aFileModeReadOnly, 
  	     aFileAreaUser, &fileRef, &err);
  if (err != aErrNone)
    return aTrue;

  aStringCopy(msg, "size, ");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  
  aFile_GetSize(ioLibRef, fileRef, &bytes, &err);
  if (err != aErrNone)
    return aTrue;
  if (bytes != 6)
    return aTrue;

  aStringCopy(msg, "read, ");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);

  aFile_Read(ioLibRef, fileRef, (char*)data, 6, &bytes, &err);
  if (err != aErrNone)
    return aTrue;
  if (bytes != 6)
    return aTrue;
  if ((data[0] != 43)
      || (data[1] != 32)
      || (data[2] != 45)
      || (data[3] != 78)
      || (data[4] != 43)
      || (data[5] != 56))
    return aTrue;

  aStringCopy(msg, "close...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);

  aFile_Close(ioLibRef, fileRef, &err);
  if (err != aErrNone)
    return aTrue;

  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aStringCopy(msg, "   Usage, delete...");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);

  aFile_Delete(ioLibRef, testusefilename, aFileAreaUser, &err); 
  if (err != aErrNone)
    return aTrue;

  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  return aFalse;
    
} /* sTest_aFile_Use */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aIO_GetMSTicks
 */

aBool sTest_aIO_GetMSTicks(aIOLib ioLibRef, aStreamRef out)
{
  aErr ioErr;
  unsigned long t1, t2;
  long i;
  double b;
  char msg[100];

  if (aIO_GetMSTicks(ioLibRef, &t1, &ioErr))
    return aTrue;
  if (ioErr != aErrNone)
    return aTrue;
  if (aIO_GetMSTicks(ioLibRef, &t1, NULL))
    return aTrue;

#ifndef aPALM
  aStringCopy(msg, "   GetMSTicks, NULL lib..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aIO_GetMSTicks(NULL, NULL, NULL))
    return aTrue;
  aIO_GetMSTicks(NULL, NULL, &ioErr); 
  if (ioErr != aErrParam)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif

  aStringCopy(msg, "   GetMSTicks, NULL time value..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (!aIO_GetMSTicks(ioLibRef, NULL, &ioErr))
    return aTrue;
  aIO_GetMSTicks(ioLibRef, NULL, &ioErr); 
  if (ioErr != aErrParam)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* waste some time */
#if defined(aPALM) || defined(aWINCE)
  for (i = 0, b = 0; i < 1000; i++) {
    b = b / 0.003;
    b = b * 23.0;
  }
#else
  for (i = 0, b = 0; i < 100000000; i++) {
    b = b / 0.003;
    b = b * 23.0;
  }
#endif
  
  aStringCopy(msg, "   GetMSTicks, time moves on..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aIO_GetMSTicks(ioLibRef, &t2, &ioErr))
    return aTrue;
  if (t1 >= t2)
    return aTrue;  
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  return aFalse;

} /* sTest_aIO_GetMSTicks */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aIO_MSSleep
 */

aBool sTest_aIO_MSSleep(aIOLib ioLibRef, aStreamRef out)
{
  aErr ioErr = aErrNone;
  char msg[100];
  unsigned long start, end;

  aStringCopy(msg, "   MSSleep, basic..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aIO_GetMSTicks(ioLibRef, &start, &ioErr))
    return aTrue;
  if (aIO_MSSleep(ioLibRef, 1000, &ioErr))
    return aTrue;
  if (aIO_GetMSTicks(ioLibRef, &end, &ioErr))
    return aTrue;
  if (end - start < 950)
    return aTrue;
  if (end - start > 1050)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aStringCopy(msg, "   MSSleep, NULL error..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aIO_GetMSTicks(ioLibRef, &start, &ioErr))
    return aTrue;  
  if (aIO_MSSleep(ioLibRef, 1000, NULL))
    return aTrue;
  if (aIO_GetMSTicks(ioLibRef, &end, &ioErr))
    return aTrue;
  if (end - start < 950)
    return aTrue;
  if (end - start > 1050)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aStringCopy(msg, "   MSSleep, no time..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aIO_GetMSTicks(ioLibRef, &start, &ioErr))
    return aTrue;  
  if (aIO_MSSleep(ioLibRef, 0, NULL))
    return aTrue;
  if (aIO_GetMSTicks(ioLibRef, &end, &ioErr))
    return aTrue;
  if (end - start > 1)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  return aFalse;

} /* sTest_aIO_MSSleep */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sTest_aTokenizer_Create
 */

aBool sTest_aTokenizer_Create(aIOLib ioLibRef, aStreamRef out)
{
  aErr err;
  char msg[100];
  aStreamRef tokenStream = 0;
  aTokenizerRef tokenizerRef;
  aFileRef file;

  aStream_WriteLine(ioLibRef, out, "  aTokenizer_Create", NULL);

#ifndef aPALM
  aStringCopy(msg, "   NULL Lib Ref..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aTokenizer_Create(NULL, tokenStream, "test", aFileAreaUser,
  	     NULL, NULL, &tokenizerRef, &err);
  if (err != aErrParam)
    return aTrue;
  if (!aTokenizer_Create(NULL, &tokenStream, "test", aFileAreaUser,
  	                 NULL, NULL, tokenizerRef, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */

  aStringCopy(msg, "   NULL Tokenizer Ref..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aTokenizer_Create(ioLibRef, tokenStream, "test", aFileAreaUser,
  	     NULL, NULL, NULL, &err);
  if (err != aErrParam)
    return aTrue;
  if (!aTokenizer_Create(ioLibRef, tokenStream, "test", aFileAreaUser,
  	                 NULL, NULL, NULL, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aStringCopy(msg, "   NULL Token Stream..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aTokenizer_Create(ioLibRef, NULL, "test", aFileAreaUser,
  	            NULL, NULL, &tokenizerRef, &err);
  if (err != aErrParam)
    return aTrue;
  if (!aTokenizer_Create(ioLibRef, NULL, "test", aFileAreaUser,
  	                 NULL, NULL, &tokenizerRef, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* create a file for testing and create a working input stream 
   * from the test file */
  
  aStringCopy(msg, "   Create temp file..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Open(ioLibRef, "tokenfile.txt", aFileModeWriteOnly, 
  	         aFileAreaTest, &file, NULL))
    return aTrue;
  if (aFile_Write(ioLibRef, file, "\n", 1, NULL, NULL))
    return aTrue;
  if (aFile_Write(ioLibRef, file, "line 2\r\n", 8, NULL, NULL))
    return aTrue;
  if (aFile_Write(ioLibRef, file, "line 3\r", 7, NULL, NULL))
    return aTrue;
  if (aFile_Write(ioLibRef, file, "line 4\n", 7, NULL, NULL))
    return aTrue;
  if (aFile_Write(ioLibRef, file, "line 5 <test>\n", 14, NULL, NULL))
    return aTrue;

  if (aFile_Close(ioLibRef, file, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aStringCopy(msg, "   Open temp input stream..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aStream_CreateFileInput(ioLibRef, "tokenfile.txt", 
  	         aFileAreaTest, &tokenStream, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aStringCopy(msg, "   Create and destroy..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  aTokenizer_Create(ioLibRef, tokenStream, "tokenfile.txt", 
  		    aFileAreaTest, NULL, NULL, &tokenizerRef, &err);
  if (err != aErrNone)
    return aTrue;
  aTokenizer_Destroy(ioLibRef, tokenizerRef, &err);
  if (err != aErrNone)
    return aTrue;
  if (aStream_CreateFileInput(ioLibRef, "tokenfile.txt", 
  	         aFileAreaTest, &tokenStream, NULL))
    return aTrue;
  if (aTokenizer_Create(ioLibRef, tokenStream, "test", 
  			 aFileAreaUser, NULL, NULL, &tokenizerRef, NULL))
    return aTrue;
  if (aTokenizer_Destroy(ioLibRef, tokenizerRef, &err))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

#if 0
  /* clean up test input file */
  aStringCopy(msg, "   Delete temp input file..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aFile_Delete(ioLibRef, "tokenfile.txt", aFileAreaTest, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif

  return(aFalse);

} /* sTest_aTokenizer_Create */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aFileTests
 */

aBool aFileTests(aIOLib ioLibRef, aStreamRef out)
{
  aStream_WriteLine(ioLibRef, out, " Basic File Tests", NULL);

  if (sTest_aFile_Open(ioLibRef, out))
    return 1;
  if (sTest_aFile_Close(ioLibRef, out))
    return 1;
  if (sTest_aFile_Read(ioLibRef, out))
    return 1;
  if (sTest_aFile_Write(ioLibRef, out))
    return 1;
  if (sTest_aFile_Seek(ioLibRef, out))
    return 1;
  if (sTest_aFile_GetSize(ioLibRef, out))
    return 1;
  if (sTest_aFile_Delete(ioLibRef, out))
    return 1;
  if (sTest_aFile_Use(ioLibRef, out))
    return 1;

  aStream_WriteLine(ioLibRef, out, " File tests complete", NULL);

  return(0);

} /* aFileTests */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStreamTests
 */

aBool aStreamTests(aIOLib ioLibRef, aStreamRef out)
{
#ifndef aPALM
  aErr err = aErrNone;
  aStreamRef streamRef;
  char readbuf[100];
  char msg[100];
#endif

  aStream_WriteLine(ioLibRef, out, " Basic Stream Tests", NULL);

#ifndef aPALM
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *
   *   aStream_Readline
   *
   */

  aStream_WriteLine(ioLibRef, out, "  aStream_ReadLine tests", NULL);

  aStringCopy(msg, "   Read lines in Windows test file..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);

  if (aStream_CreateFileInput(ioLibRef, "win32_testfile.txt", 
  			      aFileAreaTest,
			      &streamRef, &err))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, "Windows Test Data File"))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, "Line 2"))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, ""))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, "Line 4"))
    return aTrue;
  if (aStream_Destroy(ioLibRef, streamRef, &err))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aStringCopy(msg, "   Read lines in MacOS test file..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aStream_CreateFileInput(ioLibRef, "mac_testfile.txt", 
  			      aFileAreaTest, &streamRef, &err))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, "Mac Test Data File"))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, "Line 2"))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, ""))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, "Line 4"))
    return aTrue;
  if (aStream_Destroy(ioLibRef, streamRef, &err))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  aStringCopy(msg, "   Read lines in Unix test file..");
  aStream_Write(ioLibRef, out, msg, aStringLen(msg), NULL);
  if (aStream_CreateFileInput(ioLibRef, "unix_testfile.txt", 
  			      aFileAreaTest, &streamRef, &err))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, "Unix Test Data File"))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, "Line 2"))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, ""))
    return aTrue;
  if (aStream_ReadLine(ioLibRef, streamRef, readbuf, 100, &err))
    return aTrue;
  if (aStringCompare(readbuf, "Line 4"))
    return aTrue;
  if (aStream_Destroy(ioLibRef, streamRef, &err))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */

  aStream_WriteLine(ioLibRef, out, " Stream Tests Complete", NULL);

  return aFalse;

} /* aStreamTests */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aSettingFile tests
 */

aBool aSettingFileTests(aIOLib ioLibRef, aStreamRef out)
{
  aErr err = aErrNone;
  aSettingFileRef settingFileRef;
  aStreamRef testFileStream;
  int intVal;
  unsigned long ulongVal;
  float floatVal;
  char* stringVal;

  aStream_WriteLine(ioLibRef, out, " Setting File Tests", NULL);

  aStream_WriteLine(ioLibRef, out, "  aSettingFile_Create tests", NULL);

#ifndef aPALM
  sTest_Print("   Create with NULL ioLib..", out);
  if (!aSettingFile_Create(NULL, 32, "test.config", 
  			   NULL, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Create with NULL ioLib, ", out);
  if (!aSettingFile_Create(NULL, 32, "test.config", 
  			   NULL, &err))
    return aTrue;
  sTest_Print("checking error..", out);
  if (err != aErrParam)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
#endif /* aPALM */


  sTest_Print("   Create with NULL settingFileRef..", out);
  if (!aSettingFile_Create(ioLibRef, 32, "test.config", 
  			   NULL, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Create with NULL settingFileRef, ", out);
  if (!aSettingFile_Create(NULL, 32, "test.config", 
  			   NULL, &err))
    return aTrue;
  sTest_Print("checking error..", out);
  if (err != aErrParam)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  sTest_Print("   Create with NULL filename..", out);
  if (!aSettingFile_Create(ioLibRef, 32, NULL, 
  			   &settingFileRef, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Create with NULL ioLib, ", out);
  if (!aSettingFile_Create(NULL, 32, NULL, 
  			   &settingFileRef, &err))
    return aTrue;
  sTest_Print("checking error..", out);
  if (err != aErrParam)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  sTest_Print("   Create without setting file..", out);
  if (aSettingFile_Create(ioLibRef, 32, "test.config", 
  			  &settingFileRef, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get default int, ", out);
  if (aSettingFile_GetInt(ioLibRef, settingFileRef, "empty_int", 
  			  &intVal, 32, NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (intVal != 32)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get default ulong, ", out);
  if (aSettingFile_GetULong(ioLibRef, settingFileRef, "empty_ulong", 
  			    &ulongVal, 1000000, NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (ulongVal != 1000000)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get default float, ", out);
  if (aSettingFile_GetFloat(ioLibRef, settingFileRef, "empty_float", 
  			    &floatVal, 3.1459f, NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (floatVal != 3.1459f)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get default string, ", out);
  if (aSettingFile_GetString(ioLibRef, settingFileRef, "empty_string", 
  			     &stringVal, "not there", NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (aStringCompare(stringVal, "not there"))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
  
  sTest_Print("   Destroy..", out);
  if (aSettingFile_Destroy(ioLibRef, settingFileRef, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Destroying again..", out);
  if (!aSettingFile_Destroy(ioLibRef, settingFileRef, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Destroying again, ", out);
  if (!aSettingFile_Destroy(ioLibRef, settingFileRef, &err))
    return aTrue;
  sTest_Print("checking error..", out);
  if (err != aErrParam)
    return aTrue;  
  aStream_WriteLine(ioLibRef, out, "passed", NULL);


  /* here we build a small setting file for testing */
  sTest_Print("   Building test file..", out);
  if (aStream_CreateFileOutput(ioLibRef, "test.config",
  			       aFileAreaBinary, &testFileStream, NULL))
    return aTrue;
  if (aStream_WriteLine(ioLibRef, testFileStream, "# test file", NULL))
    return aTrue;
  if (aStream_WriteLine(ioLibRef, testFileStream, "intVal1 = 56", NULL))
    return aTrue;
  if (aStream_WriteLine(ioLibRef, testFileStream, "ulongVal1 = 2000000", NULL))
    return aTrue;
  if (aStream_WriteLine(ioLibRef, testFileStream, "floatVal1 = 3.4", NULL))
    return aTrue;
  if (aStream_WriteLine(ioLibRef, testFileStream, "floatVal2=7", NULL))
    return aTrue;
  if (aStream_WriteLine(ioLibRef, testFileStream, "stringVal1=test string", NULL))
    return aTrue;
  if (aStream_Destroy(ioLibRef, testFileStream, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Create with setting file..", out);
  if (aSettingFile_Create(ioLibRef, 32, "test.config", 
  			  &settingFileRef, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get default int, ", out);
  if (aSettingFile_GetInt(ioLibRef, settingFileRef, "empty_int", 
  			  &intVal, 32, NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (intVal != 32)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get existing int, ", out);
  if (aSettingFile_GetInt(ioLibRef, settingFileRef, "intVal1", 
  			  &intVal, 32, NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (intVal != 56)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get default ulong, ", out);
  if (aSettingFile_GetULong(ioLibRef, settingFileRef, "empty_ulong", 
  			    &ulongVal, 1000000, NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (ulongVal != 1000000)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get existing ulong, ", out);
  if (aSettingFile_GetULong(ioLibRef, settingFileRef, "ulongVal1", 
  			    &ulongVal, 1000000, NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (ulongVal != 2000000)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get default float, ", out);
  if (aSettingFile_GetFloat(ioLibRef, settingFileRef, "empty_float", 
  			    &floatVal, 3.1459f, NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (floatVal != 3.1459f)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get existing float, ", out);
  if (aSettingFile_GetFloat(ioLibRef, settingFileRef, "floatVal1", 
  			    &floatVal, 3.1459f, NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (floatVal != 3.4f)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get existing float, ", out);
  if (aSettingFile_GetFloat(ioLibRef, settingFileRef, "floatVal2", 
  			    &floatVal, 3.1459f, NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (floatVal != 7.0f)
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get default string, ", out);
  if (aSettingFile_GetString(ioLibRef, settingFileRef, "empty_string", 
  			     &stringVal, "not there", NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (aStringCompare(stringVal, "not there"))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Get existing string, ", out);
  if (aSettingFile_GetString(ioLibRef, settingFileRef, "stringVal1", 
  			     &stringVal, "not there", NULL))
    return aTrue;
  sTest_Print("checking..", out);
  if (aStringCompare(stringVal, "test string"))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);
  
  sTest_Print("   Destroy..", out);
  if (aSettingFile_Destroy(ioLibRef, settingFileRef, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Destroying again..", out);
  if (!aSettingFile_Destroy(ioLibRef, settingFileRef, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  sTest_Print("   Destroying again, ", out);
  if (!aSettingFile_Destroy(ioLibRef, settingFileRef, &err))
    return aTrue;
  sTest_Print("checking error..", out);
  if (err != aErrParam)
    return aTrue;  
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  /* here we build a small setting file for testing */
  sTest_Print("   Cleaning up test file..", out);
  if (aFile_Delete(ioLibRef, "test.config",
  		   aFileAreaBinary, NULL))
    return aTrue;
  aStream_WriteLine(ioLibRef, out, "passed", NULL);

  return aFalse;

} /* aSettingFileTests */


aBool aIOTests(aIOLib ioLibRef, aStreamRef outputStream)
{
  aStream_WriteLine(ioLibRef, outputStream, " Basic General IO Tests", NULL);

  if (sTest_aIO_GetMSTicks(ioLibRef, outputStream))
    return aTrue;

  if (sTest_aIO_MSSleep(ioLibRef, outputStream))
    return aTrue;

  aStream_WriteLine(ioLibRef, outputStream, " General IO tests complete", NULL);

  return aFalse;

} /* aIOTests */


aBool aTokenizerTests(aIOLib ioLibRef, aStreamRef out)
{
  aStream_WriteLine(ioLibRef, out, " Basic Tokenizer Tests", NULL);

  if (sTest_aTokenizer_Create(ioLibRef, out))
    return aTrue;

  aStream_WriteLine(ioLibRef, out, " Tokenizer tests complete", NULL);

  return aFalse;

} /* aTokenizerTests */


