/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aMemLeakDebug.c					   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifdef aLEAKCHECK

#define aMAXLEAKCHECKFILENAME	128

#include "aDefs.h"
#include "aMemLeakDebug.h"

#ifdef aMAC
#define aLeakAlloc(s)		NewPtr((long)(s))
#define aLeakFree(s)		DisposePtr(s)
#endif /* aMAC */

#ifdef aPALM
#define aLeakAlloc(s)		MemPtrNew(s)
#define aLeakFree(s)		MemPtrFree(s)
#endif /* aPALM */

#ifdef aWIN
#define aLeakAlloc(s)		malloc(s)
#define aLeakFree(s)		free(s)
#endif /* aWIN */

#ifdef aWINCE
#include <string.h>
#define aLeakAlloc(s)		malloc(s)
#define aLeakFree(s)		free(s)
#endif /* aWINCE */

#ifdef aUNIX
#define aLeakAlloc(s)		malloc(s)
#define aLeakFree(s)		free(s)
#endif /* aUNIX */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * block for tracking memory allocations
 */

typedef struct aLeakCheckBlock {
  struct aLeakCheckBlock*	pNext;
  char				filename[aMAXLEAKCHECKFILENAME];
  int				linenum;
  int				allocnum;
  aMemSize			size;
} aLeakCheckBlock;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * globals to maintain allocated and freed memory blocks
 */
 
aLeakCheckBlock* g_pLeakCheckAlloc = NULL;
aLeakCheckBlock* g_pLeakCheckFree = NULL;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aLeakCheckAlloc
 */

aMemPtr aLeakCheckAlloc(aMemSize size,
			char* filename,
			int linenum)
{
  aLeakCheckBlock* pNew;
  char* p;
  static int s_nLeakCheckAlloc = 0;

  pNew = (aLeakCheckBlock*)aLeakAlloc(sizeof(aLeakCheckBlock) + size);
  if (pNew == NULL)
    return NULL;
  
  aStringNCopy(pNew->filename, filename, aMAXLEAKCHECKFILENAME);
  pNew->linenum = linenum;
  pNew->size = size;
  pNew->allocnum = ++s_nLeakCheckAlloc;

  /* link it into the chain of allocated blocks */
  pNew->pNext = g_pLeakCheckAlloc;
  g_pLeakCheckAlloc = pNew;

  /* compute the offset to the block's data area */
  p = (char*)pNew;
  p += sizeof(aLeakCheckBlock);
  
  return(p);
  
} /* end of aLeakCheckAlloc */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aLeakCheckFree
 */

void aLeakCheckFree(aMemPtr p,
		    char* filename,
		    int linenum)
{
  char* pMem;
  aLeakCheckBlock* pRec;
  aLeakCheckBlock* pTemp;
  aLeakCheckBlock* pPrev;

  /* compute the actual block header address */
  pMem = (char*)p;
  pMem -= sizeof(aLeakCheckBlock);  
  pRec = (aLeakCheckBlock*)pMem;

  /* find the block in the allocated list */
  pPrev = NULL;
  pTemp = g_pLeakCheckAlloc;
  
  do {

    /* found it */
    if (pTemp == pRec) {
    
      /* pull it out of the list */
      if (pPrev == NULL) {
        /* special case at head of list */
        g_pLeakCheckAlloc = pRec->pNext;
      } else {
        pPrev->pNext = pTemp->pNext;
      }
      break;
    }
    /* advance to the next in the list */
    pPrev = pTemp;
    pTemp = pTemp->pNext;
  } while (pTemp != NULL);

  if (pTemp == NULL) {
    /* error, freeing memory not allocated */
    char msg[200];
    char num[10];
    aStringCopy(msg, "Freeing Unallocated Memory: ");
    aStringCat(msg, filename);
    aStringCat(msg, ", line ");
    aStringFromInt(num, linenum);
    aStringCat(msg, num);
    aDebugAlert(msg);
  }

  /* just toss the freed memory record */
  aLeakFree((aMemPtr)pTemp);

} /* end of aLeakCheckFree */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aLeakCheckCleanup
 */

void aLeakCheckCleanup(void)
{
  aLeakCheckBlock* pTemp = g_pLeakCheckAlloc;

  while(pTemp != NULL) {
    /* error, freeing memory not freed */
    char msg[200];
    char num[10];
    aStringCopy(msg, "Memory Leak: ");
    aStringCat(msg, pTemp->filename);
    aStringCat(msg, ", line ");
    aStringFromInt(num, pTemp->linenum);
    aStringCat(msg, num);
    aStringCat(msg, ", ");
    aStringFromInt(num, pTemp->size);
    aStringCat(msg, num);
    aStringCat(msg, " bytes, allocation ");
    aStringFromInt(num, pTemp->allocnum);
    aStringCat(msg, num);
    aDebugAlert(msg);
    pTemp = pTemp->pNext;
  }

} /* aLeakCheckCleanup */

#endif /* aLEAKCHECK */
