/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aModuleUtil.c						   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Implementation of BrainStem module utility         */
/*		function routines.                                 */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "aCmd.tea"
#include "aStemMsg.h"
#include "aUtil.h"
#include "aModuleUtil.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * local defines
 */

#define MAXUTILTIMEOUT	500


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * local prototypes 
 */

static aBool sReturnFilter(const unsigned char module,
		           const unsigned char dataLength,
		           const char* data,
		           void* ref);


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sReturnFilter
 */

aBool sReturnFilter(const unsigned char module,
		    const unsigned char dataLength,
		    const char* data,
		    void* ref)
{
  /* if we have a vm exit packet, get the data */
  if ((dataLength == 2) 
      && ((unsigned char)data[0] == cmdDEBUG)) {
    return aTrue;
  }

  return aFalse;

} /* sReturnFilter */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aModuleUtil_EnsureModule
 *
 * This routine ensures a live module is in place and that the 
 * heartbeat is established with the module.
 */

aBool aModuleUtil_EnsureModule(aStemLib stemRef,
			       const unsigned char address)

{
  aBool bModule = aTrue;
  char data[aSTEMMAXPACKETBYTES];
  aPacketRef packet;
  int i;

  if (!stemRef)
    bModule = aFalse;

  /* prepare the packet data */
  
  if (bModule) {
    aErr muErr = aErrNone;
    for (i = 0; (muErr == aErrNone) && (i < 3); i++) {

      /* build a debug packet */
      data[0] = cmdDEBUG;
      data[1] = (char)i;
      aPacket_Create(stemRef, address, 2,
    		     data, &packet, &muErr);

      /* send the packet */
      if (muErr == aErrNone)
        aStem_SendPacket(stemRef, packet, &muErr);

      /* now, wait for the reply packet */
      if (muErr == aErrNone)
        aStem_GetPacket(stemRef,
    		        sReturnFilter,
    		        (void*)i,
    		        MAXUTILTIMEOUT,
    		        &packet,
    		        &muErr);

      /* clean up the reply packet */
      if (muErr == aErrNone)
        aPacket_Destroy(stemRef, packet, &muErr);

      if ((i < 2) && (muErr == aErrTimeout))
        muErr = aErrNone;
    } /* for */
    if (muErr != aErrNone)
      bModule = aFalse;
  }

  return bModule;

} /* aModuleUtil_EnsureModule */

