/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aModuleVM.c						   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Definition of BrainStem module vm access routines. */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "aCmd.tea"
#include "aStemMsg.h"
#include "aUtil.h"
#include "aModuleVM.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * local defines
 */

#define MAXEXECUTETIMEOUT	1000


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * local prototypes 
 */

static aBool sReturnFilter(const unsigned char module,
		           const unsigned char dataLength,
		           const char* data,
		           void* ref);


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sReturnFilter
 */

aBool sReturnFilter(const unsigned char module,
		    const unsigned char dataLength,
		    const char* data,
		    void* ref)
{
  int *pVal = (int*)ref;

  /* if we have a vm exit packet, get the data */
  if ((dataLength >= 4) 
      && ((unsigned char)data[0] == cmdMSG)
      && (data[1] == vmExit)) {
  
    /* get the return value based on the data length */
    switch (dataLength) {
    
    /* char return value (in TEA) */
    case 5:
      *pVal = data[4];
      break;

    /* int return value (in TEA) */
    case 6:
      *pVal = aUtil_RetrieveShort(&data[4]);
      break;

    default:
      *pVal = 0;
      break;

    } /* switch */
    return aTrue;
  }

  return aFalse;

} /* sReturnFilter */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aModuleVM_Execute
 */

aErr aModuleVM_Execute(aStemLib stemRef,
		       const unsigned char module,
		       const unsigned char slot,
		       const unsigned char nParamBytes,
		       const char* pParamBytes,
		       int* pReturn)
{
  aErr moduleErr = aErrNone;
  char data[aSTEMMAXPACKETBYTES];
  aPacketRef packet;

  if (!stemRef)
    moduleErr = aErrParam;
  else if (nParamBytes > 5)
    moduleErr = aErrRange;

  /* prepare the packet data */
  if (moduleErr == aErrNone) {
    data[0] = cmdVM_RUN;
    data[1] = 3; /* first with no debug */
    data[2] = (char)slot;
    if (nParamBytes > 0)
      aMemCopy(&data[3], pParamBytes, nParamBytes);
  }

  /* build the packet */
  if (moduleErr == aErrNone) {
    aPacket_Create(stemRef, module, 
    		   (unsigned char)(3 + nParamBytes),
    		   data, &packet, &moduleErr);
  }

  /* send the packet */
  if (moduleErr == aErrNone) {
    aStem_SendPacket(stemRef,
    		     packet,
    		     &moduleErr);
  }

  /* now, wait for launch message */
  if (moduleErr == aErrNone)
    aStem_GetPacket(stemRef,
    		    NULL,
    		    NULL,
    		    MAXEXECUTETIMEOUT,
    		    &packet,
    		    &moduleErr);

  /* make sure we got a launch message */
  if (moduleErr == aErrNone) {
    unsigned char address;
    unsigned char length;
    if (!aPacket_GetData(stemRef, packet,
    		         &address, &length,
    		         data, NULL)
        && (address == module)
        && (length > 0)
        && (data[0] == cmdVM_RUN))
      aPacket_Destroy(stemRef, packet, NULL);
  }

  /* now, wait for process exit packet */
  if (moduleErr == aErrNone) {
    aStem_GetPacket(stemRef,
    		    sReturnFilter,
    		    pReturn,
    		    MAXEXECUTETIMEOUT,
    		    &packet,
    		    &moduleErr);
  }

  return moduleErr;

} /* aModuleVM_Execute */
