/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aModuleVal.c						   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Definition of BrainStem module value access        */
/*              routines.                                          */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "aCmd.tea"
#include "aStemMsg.h"
#include "aUtil.h"
#include "aModuleVal.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * local defines
 */

#define MAXVALTIMEOUT	100


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * local prototypes 
 */

static aBool sReturnFilter(const unsigned char module,
		           const unsigned char dataLength,
		           const char* data,
		           void* ref);


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sReturnFilter
 */

aBool sReturnFilter(const unsigned char module,
		    const unsigned char dataLength,
		    const char* data,
		    void* ref)
{
  char *pVal = (char*)ref;

  /* if we have a vm exit packet, get the data */
  if ((dataLength == 3) 
      && ((unsigned char)data[0] == cmdVAL)) {
    *pVal = data[2];
    return aTrue;
  }

  return aFalse;

} /* sReturnFilter */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aModuleVal_Get
 */

aErr aModuleVal_Get(aStemLib stemRef,
		     const unsigned char module,
		     const eModuleVal eVal,
		     char* pVal)
{
  aErr mvErr = aErrNone;
  char data[aSTEMMAXPACKETBYTES];
  aPacketRef packet;

  if (!stemRef)
    mvErr = aErrParam;

  /* prepare the packet data */
  if (mvErr == aErrNone) {
    data[0] = cmdVAL_GET;
    data[1] = eVal;
    aPacket_Create(stemRef, module, 2,
    		   data, &packet, &mvErr);
  }

  /* send the packet */
  if (mvErr == aErrNone) {
    aStem_SendPacket(stemRef, packet, &mvErr);
  }

  /* now, wait for the reply packet */
  if (mvErr == aErrNone) {
    aStem_GetPacket(stemRef,
    		    sReturnFilter,
    		    pVal,
    		    MAXVALTIMEOUT,
    		    &packet,
    		    &mvErr);
  }

  return mvErr;

} /* aModuleVal_Get */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aModuleVal_Set
 */

aErr aModuleVal_Set(aStemLib stemRef,
		     const unsigned char module,
		     const eModuleVal eVal,
		     const char val)
{
  aErr mvErr = aErrNone;
  char data[aSTEMMAXPACKETBYTES];
  aPacketRef packet;

  if (!stemRef)
    mvErr = aErrParam;

  /* prepare the packet data */
  if (mvErr == aErrNone) {
    data[0] = cmdVAL_SET;
    data[1] = eVal;
    data[2] = val;
    aPacket_Create(stemRef, module, 3,
    		   data, &packet, &mvErr);
  }

  /* send the packet */
  if (mvErr == aErrNone) {
    aStem_SendPacket(stemRef,
    		     packet,
    		     &mvErr);
  }

  return mvErr;

} /* aModuleVal_Set */
