/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aOSDefs.h                                                 */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: This file defines a number of common OS-Specific   */
/*              routines.  Since these routines are roughly        */
/*              equivalent but often have different syntax on      */
/*              different platforms, we #define all of them here   */
/*              to allow cross-platform coding.  This also allows  */
/*              us to implement a function missing on a specific   */
/*              platform and allows us to #define away routines    */
/*              that don't make any sense for another platform     */
/*              (such as memory handle locking on Windows or       */
/*              Unix).  Finally, this allows us to add additional  */
/*              checks (and overhead) in debug builds to check for */
/*              memory leaks, etc.                                 */
/*                                                                 */
/*              There is a section for each platform we port to.   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef _aOSDefs_H_
#define _aOSDefs_H_


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * MacOS definitions
 */

#ifdef aMAC

#include <string.h>
#include <stdio.h>
#include "macos_util.h"

#ifndef aDEBUG
#define aDebugAlert(msg)
#endif /* aDEBUG */

#define aMemCopy(dst, src, len)		memcpy(dst, src, (unsigned long)len)
#define aBZero(buffer, len)		BlockZero(buffer, len)
#define aStringCopy(dst, src)		strcpy(dst, src)
#define aStringNCopy(dst, src, n)	strncpy(dst, src, n)
#define aStringCompare(s1, s2)		strcmp(s1, s2)
#define aStringLen(s)		 	strlen(s)
#define aStringCat(s1, s2)		strcat(s1, s2)
#define aStringFromInt(s, i)		sprintf(s, "%d", i)
#define aIntFromString(pi, s)		sscanf(s, "%d", pi)

#define aMemHandle			Handle
#define aMemPtr				Ptr
#define aMemSize			long

#ifdef aLEAKCHECK
#include "aMemLeakDebug.h"
#else /* aLEAKCHECK */
#define aMemAlloc(size)			NewPtr(size)
#define aMemFree(p)			DisposePtr(p)
#endif /* aLEAKCHECK */

#define aMemHandleNew(size)		NewHandle(size)
#define aMemHandleFree(h)		DisposeHandle(h)

#define aszEOL				"\n"

#define aLIBRETURN			int

#define aLIBREF				void *

#define aH2TS(s)			s
#define aT2HS(s)			s
#define aH2TUS(s)			s
#define aT2HUS(s)			s
#define aH2TA(a)			a
#define aT2HA(a)			a

#endif /* aMAC */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Palm OS definitions
 */

#ifdef aPALM

#include <MemoryMgr.h>
#include <StringMgr.h>
#include <ErrorMgr.h>

#ifdef aDEBUG
#define aDebugAlert(msg)		ErrDisplay(msg);
#else /* aDEBUG */
#define aDebugAlert(msg)
#endif /* aDEBUG */

#define aMemCopy(dst, src, len)		MemMove(dst, src, (Int32)len)
#define aBZero(buffer, len)		MemSet(buffer, len, 0x00)
#define aStringCopy(dst, src)		StrCopy(dst, src)
#define aStringNCopy(dst, src, n)	StrNCopy(dst, src, (short)n)
#define aStringCompare(s1, s2)		StrCompare(s1, s2)
#define aStringLen(s)			StrLen(s)
#define aStringCat(s1, s2)		StrCat(s1, s2)
#define aStringFromInt(s, i)		StrIToA(s, (long)i)
#define aIntFromString(pi, s)		*pi = (int)StrAToI(s)

#define aMemHandle			MemHandle
#define aMemPtr				MemPtr
#define aMemSize			unsigned long

#ifdef aLEAKCHECK
#include "aMemLeakDebug.h"
#else /* aLEAKCHECK */
#define aMemAlloc(size)			MemPtrNew(size)
#define aMemFree(p)			MemPtrFree(p)
#endif /* aLEAKCHECK */

#define aMemHandleNew(size)		MemHandleNew(size)
#define aMemHandleFree(h)		MemHandleFree(h)
#define aMemHandleLock(h)		MemHandleLock(h)
#define aMemHandleUnlock(h)		MemHandleUnlock(h)

#define aszEOL				"\n"
#define aczEOL				'\n'

#define aLIBRETURN			Err

#define aLIBREF				UInt16

#define aH2TS(s)			s
#define aT2HS(s)			s
#define aH2TUS(s)			s
#define aT2HUS(s)			s
#define aH2TA(s)			s
#define aT2HA(s)			s

#endif /* aPALM */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Win32 definitions
 */

#ifdef aWIN

#include <stdio.h>
#include <string.h>

#ifdef aDEBUG
#define aDebugAlert(msg)		MessageBox(NULL, msg, "Debug", MB_OK);
#else /* aDEBUG */
#define aDebugAlert(msg)
#endif /* aDEBUG */

#define aMemCopy(dst, src, len)		memcpy(dst, src, len)
#define aBZero(buffer, len)		memset(buffer, 0x00, len)
#define aStringCopy(dst, src)		strcpy(dst, src)
#define aStringNCopy(dst, src, n)	strncpy(dst, src, n)
#define aStringCompare(s1, s2)		strcmp(s1, s2)
#define aStringLen(s)			strlen(s)
#define aStringCat(s1, s2)		strcat(s1, s2)
#define aStringFromInt(s, i)		sprintf(s, "%d", i)
#define aIntFromString(pi, s)		sscanf(s, "%d", pi)

#define aMemHandle			void *
#define aMemPtr				void *
#define aMemSize			unsigned int

#ifdef aLEAKCHECK
#include "aMemLeakDebug.h"
#else /* aLEAKCHECK */
#define aMemAlloc(size)			malloc(size)
#define aMemFree(p)			free(p)
#endif /* aLEAKCHECK */

#ifdef aDEBUG

#include "aMemHandleDebug.h"

#else /* aDEBUG */

#define aMemHandleNew(size)		malloc(size)
#define aMemHandleFree(h)		free(h)
#define aMemHandleLock(h)		h
#define aMemHandleUnlock(h)		h

#endif /* aDEBUG */

#define aszEOL				"\x0D\x0A"

#define aLIBRETURN			int

#define aLIBREF				void *

					/* need to swap bytes on windows */
#define aH2TS(s)			(aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aT2HS(s)			(aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aH2TUS(s)			(unsigned aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aT2HUS(s)			(unsigned aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aH2TA(s)			(tADDRESS)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aT2HA(s)			(tADDRESS)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))

#endif /* aWIN */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * WinCE definitions
 */

#ifdef aWINCE

#ifdef aDEBUG
#include "wince_util.h"
#define aDebugAlert(msg)		aWinCE_Alert(msg);
#else /* aDEBUG */
#define aDebugAlert(msg)
#endif /* aDEBUG */

#define aMemCopy(dst, src, len)		memcpy(dst, src, len)
#define aBZero(buffer, len)		memset(buffer, 0x00, len)
#define aStringCopy(dst, src)		strcpy(dst, src)
#define aStringNCopy(dst, src, n)	strncpy(dst, src, n)
#define aStringCompare(s1, s2)		strcmp(s1, s2)
#define aStringLen(s)			strlen(s)
#define aStringCat(s1, s2)		strcat(s1, s2)
#define aStringFromInt(s, i)		sprintf(s, "%d", i)
#define aIntFromString(pi, s)		sscanf(s, "%d", pi)

#define aMemHandle			void *
#define aMemPtr				void *
#define aMemSize			unsigned int

#ifdef aLEAKCHECK
#include "aMemLeakDebug.h"
#else /* aLEAKCHECK */
#define aMemAlloc(size)			malloc(size)
#define aMemFree(p)			free(p)
#endif /* aLEAKCHECK */

#ifdef aDEBUG

#include "aMemHandleDebug.h"

#else /* aDEBUG */

#define aMemHandleNew(size)		malloc(size)
#define aMemHandleFree(h)		free(h)
#define aMemHandleLock(h)		h
#define aMemHandleUnlock(h)		h

#endif /* aDEBUG */

#define aszEOL				"\r\n"

#define aLIBRETURN			int

#define aLIBREF				void *

#define aH2TS(s)			(aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aT2HS(s)			(aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aH2TUS(s)			(unsigned aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aT2HUS(s)			(unsigned aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aH2TA(s)			(tADDRESS)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aT2HA(s)			(tADDRESS)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))

#endif /* aWINCE */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * UNIX definitions
 */

#ifdef aUNIX

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define aDebugAlert(msg)		printf("ERROR! %s\n", msg)

#define aMemCopy(dst, src, len)		memcpy(dst, src, len)
#define aBZero(buffer, len)		memset(buffer, 0x00, len)
#define aStringCopy(dst, src)		strcpy(dst, src)
#define aStringNCopy(dst, src, n)	strncpy(dst, src, n)
#define aStringCompare(s1, s2)		strcmp(s1, s2)
#define aStringLen(s)			strlen(s)
#define aStringCat(s1, s2)		strcat(s1, s2)
#define aStringFromInt(s, i)		sprintf(s, "%d", (int)i)
#define aIntFromString(pi, s)		sscanf(s, "%d", pi)

#ifdef aMACX
#define aMemHandle			char**
#else
#define aMemHandle			void *
#endif
#define aMemPtr				void *
#define aMemSize			size_t

#ifdef aLEAKCHECK
#include "aMemLeakDebug.h"
#else /* aLEAKCHECK */
#define aMemAlloc(size)			malloc(size)
#define aMemFree(p)			free(p)
#endif /* aLEAKCHECK */

#ifdef aDEBUG

#include "aMemHandleDebug.h"

#else /* aDEBUG */

#define aMemHandleNew(size)		malloc(size)
#define aMemHandleFree(h)		free(h)
#define aMemHandleLock(h)		h
#define aMemHandleUnlock(h)		h

#endif /* aDEBUG */

#define aszEOL				"\n"

#define aLIBRETURN			int

#define aLIBREF				void *

#ifdef aMACX
#define aH2TS(s)			s
#define aT2HS(s)			s
#define aH2TUS(s)			s
#define aT2HUS(s)			s
#define aH2TA(s)			s
#define aT2HA(s)			s
#else /* aMACX */
#define aH2TS(s)			(aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aT2HS(s)			(aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aH2TUS(s)			(unsigned aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aT2HUS(s)			(unsigned aSHORT)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aH2TA(s)			(tADDRESS)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define aT2HA(s)			(tADDRESS)(((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#endif /* aMACX */
#endif /* aUNIX */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Common piece for aAssert implementation
 */

#ifdef aDEBUG
#include "aAssert.h"
#else
#define aAssert(exp)
#endif

#endif /* _aOSDefs_H_ */
