/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aPacket.c                                                 */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Implementation of a platform-independent BrainStem */
/*		communication layer.    			   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "aStem.h"
#include "aStemInternal.h"
#include "aPacketInternal.h"


#ifdef aNORMALSLIB

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aPacket_GetData
 */

aLIBRETURN aPacket_GetData(aStemLib stemRef,
			   const aPacketRef packetRef,
			   unsigned char* pAddress,
			   unsigned char* pLen,
			   char* data,
			   aErr* pErr)
{
  aErr stemErr = aErrNone;
  aPacket* pPacket = (aPacket*)packetRef;

  aVALIDSTEM((aStem*)stemRef);

  /* basic parameter checking */
  if ((stemRef == NULL) || 
      (pPacket == NULL) ||
      (packetRef == NULL) || 
      (data == NULL) ||
      (pLen == NULL) ||
      (pAddress == NULL))
    stemErr = aErrParam;

  if (stemErr == aErrNone) {
    aAssert(pPacket->dataSize <= aSTEMMAXPACKETBYTES);
    aAssert(!(pPacket->address & 0x01));
    *pAddress = pPacket->address;
    *pLen = pPacket->dataSize;
    aMemCopy(data, pPacket->data, pPacket->dataSize);
  }
  
  if (stemErr == aErrNone)
    stemErr = aPacketInternal_GetData((aStem*)stemRef,
    				      packetRef,
    				      pAddress,
    				      pLen,
    				      data);

  if (pErr != NULL)
    *pErr = stemErr;

  return(stemErr != aErrNone);

} /* aPacket_GetData */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aPacket_Create
 */

aLIBRETURN aPacket_Create(aStemLib stemRef, 
			  unsigned char address,
		          unsigned char nLength,
			  char* data,
			  aPacketRef* pPacketRef,
			  aErr* pErr)
{
  aErr stemErr = aErrNone;

  aVALIDSTEM((aStem*)stemRef);

  if (stemErr == aErrNone)
    stemErr = aPacketInternal_Create((aStem*)stemRef, address, 
    			             nLength, data, (aPacketRef)pPacketRef);

  if (pErr != NULL)
    *pErr = stemErr;

  return(stemErr != aErrNone);

} /* aPacket_Create */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aPacket_Format
 */

aLIBRETURN aPacket_Format(aStemLib stemRef,
			  const aPacketRef packetRef,
			  char* pBuffer,
			  const unsigned short nMaxLength,
			  aErr* pErr)
{
  aErr stemErr = aErrNone;

  aVALIDSTEM((aStem*)stemRef);

  if (stemErr == aErrNone)
    stemErr =  aPacketInternal_Format((aPacket*)packetRef,
    				      pBuffer,
    				      nMaxLength);

  if (pErr != NULL)
    *pErr = stemErr;
  
  return(stemErr != aErrNone);
 
} /* aPacket_Format */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aPacket_Destroy
 */

aLIBRETURN aPacket_Destroy(aStemLib stemRef,
			   aPacketRef packetRef,
			   aErr* pErr)
{
  aErr stemErr = aPacketInternal_Destroy((aStem*)stemRef, 
  					 (aPacket*)packetRef);

  if (pErr != NULL) {
    *pErr = stemErr;
  }
  
  return(stemErr != aErrNone);

} /* aPacket_Destroy */

#endif /* aNORMALSLIB */
