/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aPacketInternal.h                                         */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Implementation of a platform-independent BrainStem */
/*		communication layer.    			   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2000. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef _aPacketInternal_H_
#define _aPacketInternal_H_


#include "aCmd.tea"

#include "aStem.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * packet status flags
 */

#define aPacketHasAddr    0x01
#define aPacketHasSize    0x02
#define aPacketFromStem   0x04

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * type definitions
 */

typedef struct aPacket {
  unsigned char		address;  /* from/to which iic module */
  unsigned char		dataSize; /* length of following data */
  unsigned char		curSize;  /* current number of data input */
  unsigned char		status;
  char			data[aSTEMMAXPACKETBYTES];
  int			check;
  struct aPacket*	pNext;
} aPacket;

typedef struct aPacketList {
  int 			nSize;
  aMemPoolRef		packetPoolRef;
  aPacket*		pHead;
  aPacket*		pTail;
} aPacketList;

#define aPACKETCHECK  0xCEED

#define aVALIDPACKET(p) if (!p || 				   \
			 (((aPacket*)p)->check != aPACKETCHECK))   \
			   stemErr = aErrParam;

#define aPACKET_IS_HB(p) ((p) && ((p)->dataSize == 2) && ((p)->data[0] == cmdHB))

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * routine definitions
 */

void aPacket_Init(aPacket* pPacket,
		  unsigned char initflags);
aErr aPacketInternal_Create(void* vpStem,
			    unsigned char address,
			    unsigned char nLength,
	 		    char* data,
			    aPacket** ppPacket);
int aPacket_Size(aPacket* pPacket);
int aPacket_BytesSoFar(aPacket* pPacket);
aBool aPacket_Complete(aPacket* pPacket);
aErr aPacket_AddByte(aPacket* pPacket, 
		     unsigned char byte);
aErr aPacketInternal_Format(const aPacket* pPacket,
			    char* pBuffer,
			    unsigned short nMaxLength);
aErr aPacketInternal_Destroy(void* vpStem,
	          	     aPacket* pPacket);
aErr aPacketInternal_GetData(void* vpStem,
			     const aPacketRef packetRef,
			     unsigned char* pAddress,
			     unsigned char* pLen,
			     char* data);
aErr aPacketInternal_WriteToStream(aPacket* pPacket,
				   aStreamRef stream);

aErr aPacketList_Create(aMemPoolRef packetPoolRef,
			aPacketList** ppPacketList);
aErr aPacketList_AddCopy(void* vpStem,
			 aPacketList* pPacketList,
		         aPacket* pPacket);
aErr aPacketList_GetFirst(aPacketList* pPacketList,
			  aPacketFilter filterProc,
			  void* filterRef,
			  aPacket** ppPacket);
aErr aPacketList_Destroy(void* vpStem,
			 aPacketList* pPacketList);

#endif /* _aPacketInternal_H_ */

