/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aSettingFile.c						   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2001. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include "aSettingFileInternal.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aSettingFile_Create
 */

aLIBRETURN aSettingFile_Create(aIOLib ioRef,
			       const unsigned int nMaxSettingLen,
			       const char* pFileName,
			       aSettingFileRef* pSettingFileRef,
			       aErr* pErr)
{
  aErr err = aErrNone;

  if (err == aErrNone)
    err = aSettingFileInternal_Create(ioRef,
    				      nMaxSettingLen,
    				      pFileName,
    				      (aSettingFile**)pSettingFileRef);

  if (pErr != NULL)
    *pErr = err;

  return (aLIBRETURN)(err != aErrNone);

} /* aSettingFile_Create */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aSettingFile_GetInt
 */

aLIBRETURN aSettingFile_GetInt(aIOLib ioRef,
			       aSettingFileRef settingFileRef,
			       const char* key, 
			       int* pInt,
			       const int nDefault,
			       aErr* pErr)
{
  aErr err = aErrNone;

  if (err == aErrNone)
    err = aSettingFileInternal_GetInt((aSettingFile*)settingFileRef,
    				      key,
    				      pInt,
    				      nDefault);

  if (pErr != NULL)
    *pErr = err;

  return (aLIBRETURN)(err != aErrNone);

} /* aSettingFile_GetInt */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aSettingFile_GetULong
 */
			 
aLIBRETURN aSettingFile_GetULong(aIOLib ioRef,
				 aSettingFileRef settingFileRef,
			   	 const char* key, 
			   	 unsigned long* pULong,
			   	 const unsigned long nDefault,
			  	 aErr* pErr)
{
  aErr err = aErrNone;

  if (err == aErrNone)
    err = aSettingFileInternal_GetULong((aSettingFile*)settingFileRef,
    				        key,
    				        pULong,
    				        nDefault);

  if (pErr != NULL)
    *pErr = err;

  return (aLIBRETURN)(err != aErrNone);

} /* aSettingFile_GetULong */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aSettingFile_GetFloat
 */
		   
aLIBRETURN aSettingFile_GetFloat(aIOLib ioRef,
				 aSettingFileRef settingFileRef,
			   	 const char* key, 
			   	 float* pFloat,
			   	 const float fDefault,
			  	 aErr* pErr)
{
  aErr err = aErrNone;

  if (err == aErrNone)
    err = aSettingFileInternal_GetFloat((aSettingFile*)settingFileRef,
    				        key,
    				        pFloat,
    				        fDefault);

  if (pErr != NULL)
    *pErr = err;

  return (aLIBRETURN)(err != aErrNone);

} /* aSettingFile_GetFloat */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aSettingFile_GetString
 */

aLIBRETURN aSettingFile_GetString(aIOLib ioRef,
				  aSettingFileRef settingFileRef,
			    	  const char* key, 
			    	  char** ppString,
			    	  const char* pDefault,
			  	  aErr* pErr)
{
  aErr err = aErrNone;

  if (err == aErrNone)
    err = aSettingFileInternal_GetString((aSettingFile*)settingFileRef,
    				         key,
    				         ppString,
    				         pDefault);

  if (pErr != NULL)
    *pErr = err;

  return (aLIBRETURN)(err != aErrNone);

} /* aSettingFile_GetString */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aSettingFile_GetInetAddr
 */

aLIBRETURN aSettingFile_GetInetAddr(aIOLib ioRef,
				    aSettingFileRef settingFileRef,
			    	    const char* key, 
			    	    unsigned long* pInetAddr,
			    	    const unsigned long pDefault,
			  	    aErr* pErr)
{
  aErr err = aErrNone;

  if (err == aErrNone)
    err = aSettingFileInternal_GetInetAddr((aSettingFile*)settingFileRef,
    				           key,
    				           pInetAddr,
    				           pDefault);

  if (pErr != NULL)
    *pErr = err;

  return (aLIBRETURN)(err != aErrNone);

} /* aSettingFile_GetInetAddr */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aSettingFile_Destroy
 */

aLIBRETURN aSettingFile_Destroy(aIOLib ioRef,
				aSettingFileRef settingFileRef,
			  	aErr* pErr)
{
  aErr err = aErrNone;

  if (err == aErrNone)
    err = aSettingFileInternal_Destroy((aSettingFile*)settingFileRef);

  if (pErr != NULL)
    *pErr = err;

  return (aLIBRETURN)(err != aErrNone);

} /* aSettingFile_Destroy */

