/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aStem.c                                                   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Implementation of a platform-independent BrainStem */
/*		communication layer.    			   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "aStem.h"
#include "aStemInternal.h"
#include "aVersion.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStem_GetVersion
 */

aLIBRETURN aStem_GetVersion(aStemLib libRef,
			    unsigned long *pVersion,
			    aErr* pErr)
{
  aErr stemErr = aErrNone;

  if ((libRef == NULL) || (pVersion == NULL))
    stemErr = aErrParam;

  if (stemErr == aErrNone)
    *pVersion = (aVERSION_MAJOR << 24) 
    		+ (aVERSION_MINOR << 16)
    		+ aSTEM_BUILD_NUM;

  if (pErr != NULL)
    *pErr = stemErr;

  return (aLIBRETURN)(stemErr != aErrNone);

} /* aStem_GetVersion */

#ifdef aNORMALSLIB

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStem_GetLibRef
 */

aLIBRETURN aStem_GetLibRef(aStemLib* pStemRef, 
			   aErr* pErr)
{
  aErr stemErr = aStemInternal_Create((aStem**)pStemRef);

  if (pErr != NULL)
    *pErr = stemErr;

  return (stemErr != aErrNone);

} /* aStem_GetLibRef */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStem_ReleaseLibRef
 */

aLIBRETURN aStem_ReleaseLibRef(aStemLib stemRef, 
			       aErr* pErr)
{
  aErr stemErr = aStemInternal_Destroy((aStem*)stemRef);

  if (pErr != NULL)
    *pErr = stemErr;

  return (stemErr != aErrNone);

} /* aStemReleaseLibRef */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStem_SetStream
 */

aLIBRETURN aStem_SetStream(aStemLib stemRef, 
			   aStreamRef streamRef,
			   aStemStreamType streamType,
			   aErr* pErr)
{
  aErr stemErr = aStemInternal_SetStream((aStem*)stemRef, 
  					 streamRef,
  					 streamType);

  if (pErr != NULL)
    *pErr = stemErr;

  return (stemErr != aErrNone);

} /* aStem_SetStream */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStem_GetPacket
 */

aLIBRETURN aStem_GetPacket(aStemLib stemRef, 
			   aPacketFilter filterProc,
			   void* filterRef,
			   unsigned long nMSTimeout,
			   aPacketRef* pPacketRef,
			   aErr* pErr)
{
  aErr stemErr = aStemInternal_GetPacket((aStem*)stemRef, 
  					 filterProc,
  					 filterRef,
  					 nMSTimeout,
  					 (aPacket**)pPacketRef);

  if (pErr != NULL)
    *pErr = stemErr;

  return (aLIBRETURN)(stemErr != aErrNone);

} /* aStem_GetPacket */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStem_SendPacket
 */

aLIBRETURN aStem_SendPacket(aStemLib stemRef,
			    const aPacketRef packetRef,
			    aErr* pErr)
{
  aErr stemErr = aStemInternal_SendPacket((aStem*)stemRef,
  					  (aPacket*)packetRef);

  if (pErr != NULL)
    *pErr = stemErr;

  return(stemErr != aErrNone);

} /* aStem_SendPacket */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStem_DebugLine
 */

aLIBRETURN aStem_DebugLine(aStemLib stemRef,
			   const char *line,
			   aErr* pErr)
{
  aErr stemErr = aStemInternal_DebugLine((aStem*)stemRef, line);

  if (pErr != NULL)
    *pErr = stemErr;
  
  return (stemErr != aErrNone);

} /* aStem_DebugLine */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStem_CreateTEAFileInput
 */

aLIBRETURN aStem_CreateTEAFileInput(aStemLib stemRef, 
				    const unsigned char address,
				    const int nFileNumber,
				    aStreamRef* pStreamRef,
				    aErr* pErr)
{
  aErr stemErr = aStemInternal_CreateTEAFileInput((aStem*)stemRef, 
						  address,
  						  nFileNumber,
  						  pStreamRef);
  if (pErr != NULL)
    *pErr = stemErr;

  return (stemErr != aErrNone);

} /* aStem_CreateTEAFileInput */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStem_CreateTEAFileOutput
 */

aLIBRETURN aStem_CreateTEAFileOutput(aStemLib stemRef, 
				     const unsigned char address,
				     const int nFileNumber,
				     aStreamRef* pStreamRef,
				     aErr* pErr)
{
  aErr stemErr = aStemInternal_CreateTEAFileOutput((aStem*)stemRef, 
  						   address,
  						   nFileNumber,
  						   pStreamRef);
  if (pErr != NULL)
    *pErr = stemErr;
  
  return (stemErr != aErrNone);

} /* aStem_CreateTEAFileOutput */


#if 0
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aStem_LaunchVM
 */

aLIBRETURN aStem_LaunchVM(aStemLib stemRef,
			  const unsigned char module,
			  const unsigned char fileSlot,
			  const char* pProgramData,
			  const unsigned char nProgramDataLen,
			  aTEAProcessID* pPID,
			  aErr* pErr)
{
  aErr stemErr = aStemInternal_LaunchVM((aStem*)stemRef, 
  					module,
  					fileSlot,
  					pProgramData,
  					nProgramDataLen,
  					pPID);

  if (pErr != NULL)
    *pErr = stemErr;
  
  return (stemErr != aErrNone);

} /* aStem_LaunchVM */
#endif

#endif /* aNORMALSLIB */
