/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aStemInternal.h                                           */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Definition of a platform-independent BrainStem	   */
/*		communication layer.    			   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef _aStemInternal_H_
#define _aStemInternal_H_

#include "aErr.h"
#include "aDefs.h"
#include "aOSDefs.h"
#include "aPacketInternal.h"
#include "aStemOSExport.h"
#include "aIO.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * constant definitions
 */

#define aSTEMMAXDEBUGLINE		80
#define aPACKETPOOLBLOCKSIZE 		16


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * type definitions
 */

typedef struct aStem {
  aIOLib			ioRef;
  aStreamRef			debugStream;
  aStreamRef			linkStream;
  aStreamRef			relayStream;
  int				check;
  aMemPoolRef			packetPoolRef;
  aPacket			curLinkPacket;
  aPacket			curRelayPacket;
  aPacketList*			pLinkPacketList;
  aPacketList*			pRelayPacketList;
} aStem;

#define aSTEMCHECK	0xEEEE

#define aVALIDSTEM(p)	if ((!p) || 				   \
			(((aStem*)p)->check != aSTEMCHECK))	   \
			  stemErr = aErrParam;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * routine definitions
 */

aErr aStemInternal_Create(aStem** ppStem);

aErr aStemInternal_SetStream(aStem* pStem, 
			     aStreamRef streamRef,
			     aStemStreamType streamType);

aErr aStemInternal_DebugLine(aStem* pStem,
			     const char* line);

aErr aStemInternal_CreateTEAFileInput(aStem* pStem, 
				      const unsigned char address,
				      const int nFileNumber,
				      aStreamRef* pStreamRef);
aErr aStemInternal_CreateTEAFileOutput(aStem* pStem, 
				       const unsigned char address,
				       const int nFileNumber,
				       aStreamRef* pStreamRef);

aErr aStemInternal_DrainStream(aStem* pStem,
			       aStreamRef stream,
			       aPacket* pPacketStore,
			       aPacketFilter filterProc,
			       void* filterRef,
			       aBool bHandleHB,
			       unsigned char initflags,
			       aPacketList* pPacketList);

aErr aStemInternal_GetPacket(aStem* pStem,
			     aPacketFilter filterProc,
			     void* filterRef,
			     unsigned long nMSTimeout,
			     aPacket** ppPacket);

aErr aStemInternal_SendPacket(aStem* pStem,
			      aPacket* pPacket);

aErr aStemInternal_Destroy(aStem* pStem);

#endif /* _aStemInternal_H_ */
