/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* file: aUtil.c						   */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* description: Implementation of a platform-independent utilities */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                 */
/* Copyright  1997-2002. Acroname Inc.                            */
/*                                                                 */
/* This software is the property of Acroname Inc.  Any             */
/* distribution, sale, transmission, or re-use of this code is     */
/* strictly forbidden except with permission from Acroname Inc.    */
/*                                                                 */
/* To the full extent allowed by law, Acroname Inc. also excludes  */
/* for itself and its suppliers any liability, wheither based in   */
/* contract or tort (including negligence), for direct,            */
/* incidental, consequential, indirect, special, or punitive       */
/* damages of any kind, or for loss of revenue or profits, loss of */
/* business, loss of information or data, or other financial loss  */
/* arising out of or in connection with this software, even if     */
/* Acroname Inc. has been advised of the possibility of such       */
/* damages.                                                        */
/*                                                                 */
/* Acroname Inc.                                                   */
/* www.acroname.com                                                */
/* 720-564-0373                                                    */
/*                                                                 */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "aOSDefs.h"
#include "aUtil.h"



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aUtil_StoreShort
 */

void aUtil_StoreShort(char* storage, aSHORT val)
{
  aSHORT temp = aH2TS(val);
  aMemCopy(storage, &temp, sizeof(aSHORT));

} /* aUtil_StoreShort */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aUtil_RetrieveShort
 */

aSHORT aUtil_RetrieveShort(const char* storage)
{
  aSHORT temp;
  aSHORT temp1;

  aMemCopy(&temp, storage, sizeof(aSHORT));
  temp1 = aT2HS(temp);
  return temp1;

} /* aUtil_RetrieveShort */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aUtil_RetrieveUShort
 */

unsigned short aUtil_RetrieveUShort(const char* storage)
{
  unsigned short temp;
  unsigned short temp1;

  aMemCopy(&temp, storage, sizeof(unsigned short));
  temp1 = aT2HUS(temp);
  return temp1;

} /* aUtil_RetrieveUShort */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aString_GetFileRoot
 */

void aString_GetFileRoot(char* root, 
			 const char* filename)
{
  unsigned int len = (unsigned int)aStringLen(filename);
  char* p = (char*)&filename[len-1];
  
  while (p != filename) {
    if (*p == '.')
      break;
    p--;
    len--;
  }

  if (len > 1) {
    len--;
    aStringNCopy(root, filename, len);
    root[len] = 0;
  } else {
    aStringCopy(root, filename);
  }

} /* aString_GetFileRoot */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aString_GetFileExtension
 */

void aString_GetFileExtension(char* extension, 
			      const char* filename)
{
  unsigned int len = (unsigned int)aStringLen(filename);
  char* p = (char*)&filename[len-1];
  
  while (p != filename) {
    if (*p == '.')
      break;
    p--;
    len--;
  }

  if (p == filename)
    extension[0] = 0;
  else {
    aStringCopy(extension, p);
  }

} /* aString_GetFileExtension */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aString_GetFileExtension
 */

void aString_SetNumParam(char* result, 
			 const char* source, 
			 int num)
{
  char* p = (char*)source;
  char* d = (char*)result;
  char numStr[10];

  while (*p) {
    if (*p == '^') {
      char* n = numStr;
      p += 2;
      aStringFromInt(numStr, num);
      while (*n)
        *d++ = *n++;
    } else {
      *d++ = *p++;
    }
  }

  /* null terminate */
  *d = 0;

} /* aString_SetNumParam */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aString_ParseFloat
 */

float aString_ParseFloat(const char* pString)
{
  register const char* p = pString;
  float fValue = 0.0f;
  float fDivisor = 0.1f;
  int nNum;
  aBool bDecimal = aFalse;
  aBool bNegative = aFalse;

  /* parse the float */
  while (*p) {

    /* record the decimal occurance and error if already there */
    if (*p == '.') {
      if (bDecimal == aTrue)
        return 0.0f;
      bDecimal = aTrue;
    
    } else if (*p == '-') {
      if (bNegative == aTrue)
        return 0.0f;
      bNegative = aTrue;

    /* just bail on bad characters */
    } else if ((*p < '0') || (*p > '9')) {
      return 0.0f;

    /* accumulate the value */
    } else {
      nNum = *p - '0';
      if (bDecimal == aFalse) {
        fValue *= 10;
        fValue += nNum;
      } else {
        fValue += (float)nNum * fDivisor;
        fDivisor /= 10.0f;
      }
    }
    p++;
  }

  if (bNegative)
    return -fValue;
  else
    return fValue;

} /* aString_ParseFloat */


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * aString_FormatFloat
 */

void aString_FormatFloat(float fVal, char* pString)
{
  aBool bNegative = (fVal < 0) ? aTrue : aFalse;
  int nNum = (int)fVal;
  float fFraction = fVal - nNum;
  char num[10];

  *pString = 0;
  if (bNegative) {
    aStringCat(pString, "-");
    fFraction = -fFraction;
    nNum = -nNum;
  }
  aStringFromInt(num, nNum);
  aStringCat(pString, num);
  aStringCat(pString, ".");
  nNum = (int)(1000.0f * fFraction);
  aStringFromInt(num, nNum);
  aStringCat(pString, num);

} /* aString_FormatFloat */
