/* 
   Header file with useful debugging macros

   Compile program with gcc -DDEBUG 
   or use a #define DEBUG

 */

#ifndef _DEBUG_H
#define _DEBUG_H

#include <stdarg.h>

/* can turn on DEBUG here
 
   #define DEBUG N
   #define DEBUGALL 
*/ 

#define DEBUG

#ifdef DEBUG /*if DEBUG turned on, enable these macros */

/* Prints debugging msg: DPRINT("x is:%d", x) */
#define DPRINT(...) fprintf(stderr, __VA_ARGS__)

/* Print msg if condition false: DCHECK(x == 3, "x != 3, x = %d",x); */
#define DCHECK(x, ...) if(!(x)) fprintf(stderr, __VA_ARGS__)

#define ENTER_FUNCTION if(__FUNCTION__) printf("--> %s\n" __FUNCTION__); else printf("--> unknown\n");
#define EXIT_FUNCTION if (__FUNCTION__) printf("<-- %s\n" __FUNCTION__); else printf("<-- unknown\n");

/* If DEBUG level >= x, print message 
   Good for selectively printint debugging statements

   DEQ for conditionals:
   if ( DEQ(3) ){
     foo();
   }
   if DEBUGALL set, then all debugging actions taken
*/

#ifndef DEBUGALL
  #define DPRINTEQ(x,...) if(x == DEBUG) fprintf(stderr, __VA_ARGS__)
  #define DPRINTGR(x,...) if (DEBUG > x) fprintf(stderr, __VA_ARGS__)
  #define DPRINTLE(x,...) if (DEBUG < x) fprintf(stderr, __VA_ARGS__)

  #define DEQ(x) (x == DEBUG) /* use inside conditionals */
  #define DGR(x) (DEBUG > x)
  #define DLE(x) (DEBUG < x)
#else
  #define DPRINTEQ(x,...) DPRINT(__VA_ARGS__)
  #define DPRINTGR(x,...) DPRINT(__VA_ARGS__)
  #define DPRINTLE(x,...) DPRINT(__VA_ARGS__)
  #define DEQ(x) (1)    /* always true */
  #define DGR(x) (1)
  #define DLE(x) (1)
#endif /* end DEBUGALL */

#else       /* otherwise, make them blank */
#define DPRINT(...)
#define DCHECK(x, ...)
#define DPRINTEQ(x, ...)
#define DPRINTGR(x, ...)
#define DPRINTLE(x, ...)
#define DEQ(x) (0) /* always false */
#define DGR(x) (0)
#define DLE(x) (0)

#define ENTER_FUNCTION
#define EXIT_FUNCTION

#endif /* ifdef DEBUG */


#endif /* ifdef _DEBUG_H */








